@@ -22,6 +22,9 @@ void CheckForUpdates();
 
 QString GetBundlePath();
 QString GetResourcesPath();
+QString GetApplicationSupportPath();
+
+bool MigrateLegacyConfigFiles();
 
 }  // namespace mac
 
