@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,6 +15,7 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "fmpsparser.h"
 #include "song.h"
 
 #include <algorithm>
@@ -25,6 +27,7 @@
 #include <taglib/commentsframe.h>
 #include <taglib/fileref.h>
 #include <taglib/flacfile.h>
+#include <taglib/id3v1genres.h>
 #include <taglib/id3v2tag.h>
 #include <taglib/mp4file.h>
 #include <taglib/mpcfile.h>
@@ -88,7 +91,7 @@ const QStringList Song::kColumns = QStringList()
     << "mtime" << "ctime" << "filesize" << "sampler" << "art_automatic"
     << "art_manual" << "filetype" << "playcount" << "lastplayed" << "rating"
     << "forced_compilation_on" << "forced_compilation_off"
-    << "effective_compilation";
+    << "effective_compilation" << "skipcount" << "score";
 
 const QString Song::kColumnSpec = Song::kColumns.join(", ");
 const QString Song::kBindSpec = Prepend(":", Song::kColumns).join(", ");
@@ -148,6 +151,11 @@ Song::Private::Private()
     sampler_(false),
     forced_compilation_on_(false),
     forced_compilation_off_(false),
+    rating_(-1.0),
+    playcount_(0),
+    skipcount_(0),
+    lastplayed_(-1),
+    score_(0),
     length_(-1),
     bitrate_(-1),
     samplerate_(-1),
@@ -155,7 +163,8 @@ Song::Private::Private()
     mtime_(-1),
     ctime_(-1),
     filesize_(-1),
-    filetype_(Type_Unknown)
+    filetype_(Type_Unknown),
+    init_from_file_(false)
 {
 }
 
@@ -188,6 +197,10 @@ void Song::Init(const QString& title, const QString& artist, const QString& albu
   d->length_ = length;
 }
 
+void Song::set_genre(int id) {
+  set_genre(TStringToQString(TagLib::ID3v1::genre(id)));
+}
+
 QString Song::Decode(const TagLib::String& tag, const QTextCodec* codec) {
   if (codec && tag.isLatin1()) {  // Never override UTF-8.
     const std::string fixed = QString::fromUtf8(tag.toCString(true)).toStdString();
@@ -219,6 +232,8 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
   if(fileref->isNull())
     return;
 
+  d->init_from_file_ = true;
+
   QFileInfo info(filename);
   d->basefilename_ = info.fileName();
   d->filesize_ = info.size();
@@ -276,6 +291,17 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
           break;
         }
       }
+
+      // Parse FMPS frames
+      for (int i=0 ; i<map["TXXX"].size() ; ++i) {
+        const TagLib::ID3v2::UserTextIdentificationFrame* frame =
+            dynamic_cast<const TagLib::ID3v2::UserTextIdentificationFrame*>(map["TXXX"][i]);
+
+        if (frame->description().startsWith("FMPS_")) {
+          ParseFMPSFrame(TStringToQString(frame->description()),
+                         TStringToQString(frame->fieldList()[1]));
+        }
+      }
     }
   } else if (TagLib::Ogg::Vorbis::File* file = dynamic_cast<TagLib::Ogg::Vorbis::File*>(fileref->file())) {
     if (file->tag()) {
@@ -319,6 +345,41 @@ void Song::InitFromFile(const QString& filename, int directory_id) {
   GuessFileType(fileref.get());
 }
 
+void Song::ParseFMPSFrame(const QString& name, const QString& value) {
+  FMPSParser parser;
+  if (!parser.Parse(value) || parser.is_empty())
+    return;
+
+  QVariant var;
+  if (name == "FMPS_Rating") {
+    var = parser.result()[0][0];
+    if (var.type() == QVariant::Double) {
+      d->rating_ = var.toDouble();
+    }
+  } else if (name == "FMPS_Rating_User") {
+    // Take a user rating only if there's no rating already set
+    if (d->rating_ == -1 && parser.result()[0].count() >= 2) {
+      var = parser.result()[0][1];
+      if (var.type() == QVariant::Double) {
+        d->rating_ = var.toDouble();
+      }
+    }
+  } else if (name == "FMPS_PlayCount") {
+    var = parser.result()[0][0];
+    if (var.type() == QVariant::Double) {
+      d->playcount_ = var.toDouble();
+    }
+  } else if (name == "FMPS_PlayCount_User") {
+    // Take a user rating only if there's no playcount already set
+    if (d->rating_ == -1 && parser.result()[0].count() >= 2) {
+      var = parser.result()[0][1];
+      if (var.type() == QVariant::Double) {
+        d->playcount_ = var.toDouble();
+      }
+    }
+  }
+}
+
 void Song::ParseOggTag(const TagLib::Ogg::FieldListMap& map, const QTextCodec* codec,
                        QString* disc, QString* compilation) {
   if (!map["COMPOSER"].isEmpty())
@@ -369,6 +430,8 @@ void Song::GuessFileType(TagLib::FileRef* fileref) {
 void Song::InitFromQuery(const SqlRow& q, int col) {
   d->valid_ = true;
 
+  d->init_from_file_ = true;
+
   #define tostr(n) (q.value(n).isNull() ? QString::null : q.value(n).toString())
   #define toint(n) (q.value(n).isNull() ? -1 : q.value(n).toInt())
   #define tofloat(n) (q.value(n).isNull() ? -1 : q.value(n).toDouble())
@@ -404,15 +467,19 @@ void Song::InitFromQuery(const SqlRow& q, int col) {
   d->art_manual_ = q.value(col + 23).toString();
 
   d->filetype_ = FileType(q.value(col + 24).toInt());
-  // playcount = 25
-  // lastplayed = 26
-  // rating = 27
+  d->playcount_ = q.value(col + 25).isNull() ? 0 : q.value(col + 25).toInt();
+  d->lastplayed_ = toint(col + 26);
+  d->rating_ = tofloat(col + 27);
 
   d->forced_compilation_on_ = q.value(col + 28).toBool();
   d->forced_compilation_off_ = q.value(col + 29).toBool();
 
   // effective_compilation = 30
 
+  d->skipcount_ = q.value(col + 31).isNull() ? 0 : q.value(col + 31).toInt();
+
+  d->score_ = q.value(col + 32).isNull() ? 0 : q.value(col + 32).toInt();
+
   #undef tostr
   #undef toint
   #undef tofloat
@@ -452,6 +519,10 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     d->ctime_ = track->time_added;
     d->filesize_ = track->size;
     d->filetype_ = track->type2 ? Type_Mpeg : Type_Mp4;
+    d->rating_ = float(track->rating) / 100; // 100 = 20 * 5 stars
+    d->playcount_ = track->playcount;
+    d->skipcount_ = track->skipcount;
+    d->lastplayed_ = track->time_played;
 
     d->filename_ = QString::fromLocal8Bit(track->ipod_path);
     d->filename_.replace(':', '/');
@@ -480,6 +551,10 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     track->type1 = 0;
     track->type2 = d->filetype_ == Type_Mp4 ? 0 : 1;
     track->mediatype = 1; // Audio
+    track->rating = d->rating_ * 100; // 100 = 20 * 5 stars
+    track->playcount = d->playcount_;
+    track->skipcount = d->skipcount_;
+    track->time_played = d->lastplayed_;
   }
 #endif
 
@@ -503,6 +578,9 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     d->mtime_ = track->modificationdate;
     d->ctime_ = track->modificationdate;
 
+    d->rating_ = float(track->rating) / 100;
+    d->playcount_ = track->usecount;
+
     switch (track->filetype) {
       case LIBMTP_FILETYPE_WAV:  d->filetype_ = Type_Wav;       break;
       case LIBMTP_FILETYPE_MP3:  d->filetype_ = Type_Mpeg;      break;
@@ -539,8 +617,8 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
     track->wavecodec = 0;
     track->bitrate = d->bitrate_;
     track->bitratetype = 0;
-    track->rating = 0;
-    track->usecount = 0;
+    track->rating = d->rating_ * 100;
+    track->usecount = d->playcount_;
     track->filesize = d->filesize_;
     track->modificationdate = d->mtime_;
 
@@ -788,6 +866,11 @@ void Song::InitFromLastFM(const lastfm::Track& track) {
 void Song::MergeFromSimpleMetaBundle(const Engine::SimpleMetaBundle &bundle) {
   d->valid_ = true;
 
+  if (d->init_from_file_) {
+    // This Song was already loaded using taglib. Our tags are probably better than the engine's.
+    return;
+  }
+
   UniversalEncodingHandler detector(NS_FILTER_NON_CJK);
   QTextCodec* codec = detector.Guess(bundle);
 
@@ -827,7 +910,7 @@ void Song::BindToQuery(QSqlQuery *query) const {
   query->bindValue(":bitrate", intval(d->bitrate_));
   query->bindValue(":samplerate", intval(d->samplerate_));
 
-  query->bindValue(":directory_id", notnullintval(d->directory_id_));
+  query->bindValue(":directory", notnullintval(d->directory_id_));
   query->bindValue(":filename", d->filename_);
   query->bindValue(":mtime", notnullintval(d->mtime_));
   query->bindValue(":ctime", notnullintval(d->ctime_));
@@ -838,15 +921,19 @@ void Song::BindToQuery(QSqlQuery *query) const {
   query->bindValue(":art_manual", d->art_manual_);
 
   query->bindValue(":filetype", d->filetype_);
-  query->bindValue(":playcount", 0); // TODO
-  query->bindValue(":lastplayed", -1); // TODO
-  query->bindValue(":rating", -1);
+  query->bindValue(":playcount", d->playcount_);
+  query->bindValue(":lastplayed", intval(d->lastplayed_));
+  query->bindValue(":rating", intval(d->rating_));
 
   query->bindValue(":forced_compilation_on", d->forced_compilation_on_ ? 1 : 0);
   query->bindValue(":forced_compilation_off", d->forced_compilation_off_ ? 1 : 0);
 
   query->bindValue(":effective_compilation", is_compilation() ? 1 : 0);
 
+  query->bindValue(":skipcount", d->skipcount_);
+
+  query->bindValue(":score", d->score_);
+
   #undef intval
   #undef notnullintval
 }
@@ -872,18 +959,6 @@ void Song::ToLastFM(lastfm::Track* track) const {
   mtrack.setSource(lastfm::Track::Player);
 }
 
-QString Song::PrettyTitleWithArtist() const {
-  QString title(d->title_);
-
-  if (title.isEmpty())
-    title = QFileInfo(d->filename_).baseName();
-
-  if (is_compilation() && !d->artist_.isEmpty() && !d->artist_.toLower().contains("various"))
-    title = d->artist_ + " - " + title;
-
-  return title;
-}
-
 QString Song::PrettyTitle() const {
   QString title(d->title_);
 
@@ -893,6 +968,18 @@ QString Song::PrettyTitle() const {
   return title;
 }
 
+QString Song::PrettyTitleWithArtist() const {
+  QString title(d->title_);
+
+  if (title.isEmpty())
+    title = QFileInfo(d->filename_).baseName();
+
+  if (!d->artist_.isEmpty())
+    title = d->artist_ + " - " + title;
+
+  return title;
+}
+
 QString Song::PrettyLength() const {
   if (d->length_ == -1)
     return QString::null;
@@ -900,6 +987,25 @@ QString Song::PrettyLength() const {
   return Utilities::PrettyTime(d->length_);
 }
 
+QString Song::PrettyYear() const {
+  if (d->year_ == -1)
+    return QString::null;
+
+  return QString::number(d->year_);
+}
+
+QString Song::TitleWithCompilationArtist() const {
+  QString title(d->title_);
+
+  if (title.isEmpty())
+    title = QFileInfo(d->filename_).baseName();
+
+  if (is_compilation() && !d->artist_.isEmpty() && !d->artist_.toLower().contains("various"))
+    title = d->artist_ + " - " + title;
+
+  return title;
+}
+
 bool Song::IsMetadataEqual(const Song& other) const {
   return d->title_ == other.d->title_ &&
          d->album_ == other.d->album_ &&
