@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -157,6 +158,11 @@ class Song {
     return (d->compilation_ || d->sampler_ || d->forced_compilation_on_)
             && ! d->forced_compilation_off_;
   }
+  float rating() const { return d->rating_; }
+  int playcount() const { return d->playcount_; }
+  int skipcount() const { return d->skipcount_; }
+  int lastplayed() const { return d->lastplayed_; }
+  int score() const { return d->score_; }
 
   int length() const { return d->length_; }
   int bitrate() const { return d->bitrate_; }
@@ -179,6 +185,9 @@ class Song {
   QString PrettyTitle() const;
   QString PrettyTitleWithArtist() const;
   QString PrettyLength() const;
+  QString PrettyYear() const;
+
+  QString TitleWithCompilationArtist() const;
 
   // Setters
   bool IsEditable() const;
@@ -198,6 +207,7 @@ class Song {
   void set_bpm(float v) { d->bpm_ = v; }
   void set_year(int v) { d->year_ = v; }
   void set_genre(const QString& v) { d->genre_ = v; }
+  void set_genre(int id);
   void set_comment(const QString& v) { d->comment_ = v; }
   void set_compilation(bool v) { d->compilation_ = v; }
   void set_sampler(bool v) { d->sampler_ = v; }
@@ -213,6 +223,11 @@ class Song {
   void set_image(const QImage& i) { d->image_ = i; }
   void set_forced_compilation_on(bool v) { d->forced_compilation_on_ = v; }
   void set_forced_compilation_off(bool v) { d->forced_compilation_off_ = v; }
+  void set_rating(float v) { d->rating_ = v; }
+  void set_playcount(int v) { d->playcount_ = v; }
+  void set_skipcount(int v) { d->skipcount_ = v; }
+  void set_lastplayed(int v) { d->lastplayed_ = v; }
+  void set_score(int v) { d->score_ = qBound(0, v, 100); }
 
   // Setters that should only be used by tests
   void set_filename(const QString& v) { d->filename_ = v; }
@@ -229,6 +244,7 @@ class Song {
   // Helper methods for taglib
   static void SetTextFrame(const QString& id, const QString& value,
                            TagLib::ID3v2::Tag* tag);
+  void ParseFMPSFrame(const QString& name, const QString& value);
 
  private:
   struct Private : public QSharedData {
@@ -253,7 +269,13 @@ class Song {
     bool forced_compilation_on_;  // Set by the user
     bool forced_compilation_off_; // Set by the user
 
-    int length_;
+    float rating_;
+    int playcount_;
+    int skipcount_;
+    int lastplayed_;
+    int score_;
+
+    int length_;  // Seconds.
     int bitrate_;
     int samplerate_;
 
@@ -270,6 +292,9 @@ class Song {
     QString art_manual_;    // Set by the user - should take priority
 
     QImage image_;
+
+    // Whether this song was loaded from a file using taglib.
+    bool init_from_file_;
   };
 
   void ParseOggTag(const TagLib::Ogg::FieldListMap& map, const QTextCodec* codec, QString* disc, QString* compilation);
