@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -70,7 +71,7 @@ private:
     Finished,
   };
 
-  Result LoadLocal(const QString& filename, bool block = false);
+  Result LoadLocal(const QString& filename, bool block = false, bool ignore_playlists = false);
   void LoadLocalDirectory(const QString& filename);
   void LoadPlaylist(ParserBase* parser, const QString& filename);
   void LoadLocalDirectoryAndEmit(const QString& filename);
