@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -43,7 +44,8 @@ bool GioLister::DeviceInfo::is_suitable() const {
   return filesystem_type != "udf" &&
          filesystem_type != "smb" &&
          filesystem_type != "cifs" &&
-         filesystem_type != "ssh";
+         filesystem_type != "ssh" &&
+         filesystem_type != "isofs";
 }
 
 template <typename T, typename F>
