@@ -25,6 +25,7 @@
 #include "gstengine.h"
 #include "gstenginepipeline.h"
 #include "core/boundfuturewatcher.h"
+#include "core/utilities.h"
 
 #ifdef HAVE_IMOBILEDEVICE
 # include "gstafcsrc/gstafcsrc.h"
@@ -73,6 +74,7 @@ GstEngine::GstEngine()
     rg_mode_(0),
     rg_preamp_(0.0),
     rg_compression_(true),
+    buffer_duration_ms_(1000),
     seek_timer_(new QTimer(this)),
     timer_id_(-1),
     next_element_id_(0)
@@ -119,9 +121,7 @@ bool GstEngine::Init() {
 #endif
 
 #if defined(Q_OS_WIN32) || defined(Q_OS_DARWIN)
-  registry_filename = QString("%1/.config/%2/gst-registry-%3.bin").arg(
-      QDir::homePath(), QCoreApplication::organizationName(),
-      QCoreApplication::applicationVersion());
+  registry_filename = Utilities::GetConfigPath(Utilities::Path_GstreamerRegistry);
 #endif
 
   if (!scanner_path.isEmpty())
@@ -166,6 +166,8 @@ void GstEngine::ReloadSettings() {
   rg_mode_ = s.value("rgmode", 0).toInt();
   rg_preamp_ = s.value("rgpreamp", 0.0).toDouble();
   rg_compression_ = s.value("rgcompression", true).toBool();
+
+  buffer_duration_ms_ = s.value("bufferduration", 1000).toInt();
 }
 
 
@@ -496,6 +498,9 @@ void GstEngine::StartFadeout() {
 bool GstEngine::Play( uint offset ) {
   EnsureInitialised();
 
+  if (!current_pipeline_)
+    return false;
+
   QFuture<GstStateChangeReturn> future = current_pipeline_->SetState(GST_STATE_PLAYING);
   BoundFutureWatcher<GstStateChangeReturn, uint>* watcher =
       new BoundFutureWatcher<GstStateChangeReturn, uint>(offset, this);
@@ -739,6 +744,7 @@ shared_ptr<GstEnginePipeline> GstEngine::CreatePipeline() {
   shared_ptr<GstEnginePipeline> ret(new GstEnginePipeline(this));
   ret->set_output_device(sink_, device_);
   ret->set_replaygain(rg_enabled_, rg_mode_, rg_preamp_, rg_compression_);
+  ret->set_buffer_duration_ms(buffer_duration_ms_);
 
   ret->AddBufferConsumer(this);
   foreach (BufferConsumer* consumer, buffer_consumers_)
@@ -755,6 +761,12 @@ shared_ptr<GstEnginePipeline> GstEngine::CreatePipeline() {
 
 shared_ptr<GstEnginePipeline> GstEngine::CreatePipeline(const QUrl& url) {
   shared_ptr<GstEnginePipeline> ret = CreatePipeline();
+
+  if (url.scheme() == "hypnotoad") {
+    ret->InitFromString(kHypnotoadPipeline);
+    return ret;
+  }
+
   if (!ret->InitFromUrl(url))
     ret.reset();
 
@@ -824,16 +836,33 @@ int GstEngine::AddBackgroundStream(shared_ptr<GstEnginePipeline> pipeline) {
   disconnect(pipeline.get(), SIGNAL(MetadataFound(Engine::SimpleMetaBundle)), this, 0);
   disconnect(pipeline.get(), SIGNAL(EndOfStreamReached(bool)), this, 0);
   connect(pipeline.get(), SIGNAL(EndOfStreamReached(bool)), SLOT(BackgroundStreamFinished()));
-  if (!pipeline->SetState(GST_STATE_PLAYING)) {
-    qWarning() << "Could not set thread to PLAYING.";
-    pipeline.reset();
-    return -1;
-  }
+
   const int stream_id = next_background_stream_id_++;
   background_streams_[stream_id] = pipeline;
+
+  QFuture<GstStateChangeReturn> future = pipeline->SetState(GST_STATE_PLAYING);
+  BoundFutureWatcher<GstStateChangeReturn, int>* watcher =
+      new BoundFutureWatcher<GstStateChangeReturn, int>(stream_id, this);
+  watcher->setFuture(future);
+  connect(watcher, SIGNAL(finished()), SLOT(BackgroundStreamPlayDone()));
+
   return stream_id;
 }
 
+void GstEngine::BackgroundStreamPlayDone() {
+  BoundFutureWatcher<GstStateChangeReturn, int>* watcher =
+      static_cast<BoundFutureWatcher<GstStateChangeReturn, int>*>(sender());
+  watcher->deleteLater();
+
+  const int stream_id = watcher->data();
+  GstStateChangeReturn ret = watcher->result();
+
+  if (ret == GST_STATE_CHANGE_FAILURE) {
+    qWarning() << "Could not set thread to PLAYING.";
+    background_streams_.remove(stream_id);
+  }
+}
+
 int GstEngine::AddBackgroundStream(const QUrl& url) {
   shared_ptr<GstEnginePipeline> pipeline = CreatePipeline(url);
   if (!pipeline) {
@@ -853,12 +882,8 @@ void GstEngine::BackgroundStreamFinished() {
   pipeline->SetNextUrl(pipeline->url());
 }
 
-int GstEngine::AllGloryToTheHypnotoad() {
-  shared_ptr<GstEnginePipeline> pipeline = CreatePipeline();
-  pipeline->InitFromString(kHypnotoadPipeline);
-  if (!pipeline) {
-    return -1;
-  }
-  pipeline->SetVolume(5);  // Hypnotoad is *loud*.
-  return AddBackgroundStream(pipeline);
+void GstEngine::SetBackgroundStreamVolume(int id, int volume) {
+  shared_ptr<GstEnginePipeline> pipeline = background_streams_[id];
+  Q_ASSERT(pipeline);
+  pipeline->SetVolume(volume);
 }
