@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,12 +20,19 @@
 #include "sqlrow.h"
 #include "core/database.h"
 #include "core/scopedtransaction.h"
+#include "smartplaylists/search.h"
 
 #include <QDir>
 #include <QVariant>
 #include <QSettings>
 #include <QtDebug>
 #include <QCoreApplication>
+#include <QDateTime>
+
+const char* LibraryBackend::kNewScoreSql =
+    "case when playcount <= 0 then (%1 * 100 + score) / 2"
+    "     else (score * (playcount + skipcount) + %1 * 100) / (playcount + skipcount + 1)"
+    " end";
 
 LibraryBackend::LibraryBackend(QObject *parent)
   : LibraryBackendInterface(parent)
@@ -49,6 +57,21 @@ void LibraryBackend::UpdateTotalSongCountAsync() {
   metaObject()->invokeMethod(this, "UpdateTotalSongCount", Qt::QueuedConnection);
 }
 
+void LibraryBackend::IncrementPlayCountAsync(int id) {
+  metaObject()->invokeMethod(this, "IncrementPlayCount", Qt::QueuedConnection,
+                             Q_ARG(int, id));
+}
+
+void LibraryBackend::IncrementSkipCountAsync(int id, float progress) {
+  metaObject()->invokeMethod(this, "IncrementSkipCount", Qt::QueuedConnection,
+                             Q_ARG(int, id), Q_ARG(float, progress));
+}
+
+void LibraryBackend::UpdateSongRatingAsync(int id, float rating) {
+  metaObject()->invokeMethod(this, "UpdateSongRating", Qt::QueuedConnection,
+                             Q_ARG(int, id), Q_ARG(float, rating));
+}
+
 void LibraryBackend::LoadDirectories() {
   DirectoryList dirs = GetAllDirectories();
 
@@ -299,15 +322,18 @@ void LibraryBackend::AddOrUpdateSongs(const SongList& songs) {
         continue; // Directory didn't exist
     }
 
-
     if (song.id() == -1) {
       // Create
+
+      // Insert the row and create a new ID
       song.BindToQuery(&add_song);
       add_song.exec();
       if (db_->CheckErrors(add_song.lastError())) continue;
 
+      // Get the new ID
       const int id = add_song.lastInsertId().toInt();
 
+      // Add to the FTS index
       add_song_fts.bindValue(":id", id);
       song.BindToFtsQuery(&add_song_fts);
       add_song_fts.exec();
@@ -454,18 +480,74 @@ SongList LibraryBackend::GetSongs(const QString& artist, const QString& album, c
 Song LibraryBackend::GetSongById(int id) {
   QMutexLocker l(db_->Mutex());
   QSqlDatabase db(db_->Connect());
+  return GetSongById(id, db);
+}
+
+SongList LibraryBackend::GetSongsById(const QList<int>& ids) {
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  QStringList str_ids;
+  foreach (int id, ids) {
+    str_ids << QString::number(id);
+  }
+
+  return GetSongsById(str_ids, db);
+}
+
+SongList LibraryBackend::GetSongsById(const QStringList& ids) {
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  return GetSongsById(ids, db);
+}
+
+SongList LibraryBackend::GetSongsByForeignId(
+    const QStringList& ids, const QString& table, const QString& column) {
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  QString in = ids.join(",");
+
+  QSqlQuery q(QString("SELECT %2.ROWID, " + Song::kColumnSpec + ", %2.%3"
+                      " FROM %2, %1"
+                      " WHERE %2.%3 IN (%4) AND %1.ROWID = %2.ROWID")
+              .arg(songs_table_, table, column, in), db);
+  q.exec();
+  if (db_->CheckErrors(q.lastError())) return SongList();
+
+  QVector<Song> ret(ids.count());
+  while (q.next()) {
+    const QString foreign_id = q.value(Song::kColumns.count() + 1).toString();
+    const int index = ids.indexOf(foreign_id);
+    if (index == -1)
+      continue;
+
+    ret[index].InitFromQuery(q);
+  }
+  return ret.toList();
+}
+
+Song LibraryBackend::GetSongById(int id, QSqlDatabase& db) {
+  SongList list = GetSongsById(QStringList() << QString::number(id), db);
+  if (list.isEmpty())
+    return Song();
+  return list.first();
+}
+
+SongList LibraryBackend::GetSongsById(const QStringList& ids, QSqlDatabase& db) {
+  QString in = ids.join(",");
 
   QSqlQuery q(QString("SELECT ROWID, " + Song::kColumnSpec + " FROM %1"
-                      " WHERE ROWID = :id").arg(songs_table_), db);
-  q.bindValue(":id", id);
+                      " WHERE ROWID IN (%2)").arg(songs_table_, in), db);
   q.exec();
-  if (db_->CheckErrors(q.lastError())) return Song();
+  if (db_->CheckErrors(q.lastError())) return SongList();
 
-  q.next();
-
-  Song ret;
-  if (q.isValid()) {
-    ret.InitFromQuery(q);
+  SongList ret;
+  while (q.next()) {
+    Song song;
+    song.InitFromQuery(q);
+    ret << song;
   }
   return ret;
 }
@@ -787,3 +869,108 @@ void LibraryBackend::ForceCompilation(const QString& artist, const QString& albu
 bool LibraryBackend::ExecQuery(LibraryQuery *q) {
   return !db_->CheckErrors(q->Exec(db_->Connect(), songs_table_, fts_table_));
 }
+
+SongList LibraryBackend::FindSongs(const smart_playlists::Search& search) {
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  // Build the query
+  QString sql = search.ToSql(songs_table());
+
+  // Run the query
+  SongList ret;
+  QSqlQuery query(sql, db);
+  query.exec();
+  if (db_->CheckErrors(query.lastError()))
+    return ret;
+
+  // Read the results
+  while (query.next()) {
+    Song song;
+    song.InitFromQuery(query);
+    ret << song;
+  }
+  return ret;
+}
+
+void LibraryBackend::IncrementPlayCount(int id) {
+  if (id == -1)
+    return;
+
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  QSqlQuery q(QString("UPDATE %1 SET playcount = playcount + 1,"
+                      "              lastplayed = :now,"
+                      "              score = " + QString(kNewScoreSql).arg("1.0") +
+                      " WHERE ROWID = :id").arg(songs_table_), db);
+  q.bindValue(":now", QDateTime::currentDateTime().toTime_t());
+  q.bindValue(":id", id);
+  q.exec();
+  if (db_->CheckErrors(q.lastError()))
+    return;
+
+  Song new_song = GetSongById(id, db);
+  emit SongsStatisticsChanged(SongList() << new_song);
+}
+
+void LibraryBackend::IncrementSkipCount(int id, float progress) {
+  if (id == -1)
+    return;
+  progress = qBound(0.0f, progress, 1.0f);
+
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  QSqlQuery q(QString("UPDATE %1 SET skipcount = skipcount + 1,"
+                      "              score = " + QString(kNewScoreSql).arg(progress) +
+                      " WHERE ROWID = :id").arg(songs_table_), db);
+  q.bindValue(":id", id);
+  q.exec();
+  if (db_->CheckErrors(q.lastError()))
+    return;
+
+  Song new_song = GetSongById(id, db);
+  emit SongsStatisticsChanged(SongList() << new_song);
+}
+
+void LibraryBackend::UpdateSongRating(int id, float rating) {
+  if (id == -1)
+    return;
+
+  QMutexLocker l(db_->Mutex());
+  QSqlDatabase db(db_->Connect());
+
+  QSqlQuery q(QString("UPDATE %1 SET rating = :rating"
+                      " WHERE ROWID = :id").arg(songs_table_), db);
+  q.bindValue(":rating", rating);
+  q.bindValue(":id", id);
+  q.exec();
+  if (db_->CheckErrors(q.lastError()))
+    return;
+
+  Song new_song = GetSongById(id, db);
+  emit SongsStatisticsChanged(SongList() << new_song);
+}
+
+void LibraryBackend::DeleteAll() {
+  {
+    QMutexLocker l(db_->Mutex());
+    QSqlDatabase db(db_->Connect());
+    ScopedTransaction t(&db);
+
+    QSqlQuery q("DELETE FROM " + songs_table_, db);
+    q.exec();
+    if (db_->CheckErrors(q.lastError()))
+      return;
+
+    q = QSqlQuery("DELETE FROM " + fts_table_, db);
+    q.exec();
+    if (db_->CheckErrors(q.lastError()))
+      return;
+
+    t.Commit();
+  }
+
+  emit DatabaseReset();
+}
