@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -39,6 +40,8 @@ class LibraryFilterWidget : public QWidget {
   LibraryFilterWidget(QWidget* parent = 0);
   ~LibraryFilterWidget();
 
+  static const int kFilterDelay = 500; // msec
+
   void SetFilterHint(const QString& hint);
   void SetAgeFilterEnabled(bool enabled);
   void SetGroupByEnabled(bool enabled);
@@ -63,6 +66,9 @@ class LibraryFilterWidget : public QWidget {
   void GroupByClicked(QAction* action);
   void ClearFilter();
 
+  void FilterTextChanged(const QString& text);
+  void FilterDelayTimeout();
+
  private:
   Ui_LibraryFilterWidget* ui_;
   LibraryModel* model_;
@@ -76,6 +82,8 @@ class LibraryFilterWidget : public QWidget {
   QActionGroup* group_by_group_;
   QSignalMapper* filter_age_mapper_;
 
+  QTimer* filter_delay_;
+
   QString settings_group_;
 
   LineEditInterface* filter_;
