@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +31,8 @@ class LibraryItem : public SimpleTreeItem<LibraryItem> {
     Type_Divider,
     Type_Container,
     Type_Song,
+    Type_PlaylistContainer,
+    Type_SmartPlaylist,
   };
 
   LibraryItem(SimpleTreeModel<LibraryItem>* model)
@@ -39,6 +42,7 @@ class LibraryItem : public SimpleTreeItem<LibraryItem> {
 
   int container_level;
   Song metadata;
+  QByteArray smart_playlist_data;
 };
 
 #endif // LIBRARYITEM_H
