@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,22 +22,45 @@
 #include "sqlrow.h"
 #include "core/database.h"
 #include "playlist/songmimedata.h"
+#include "smartplaylists/generator.h"
+#include "smartplaylists/generatormimedata.h"
+#include "smartplaylists/querygenerator.h"
 #include "ui/iconloader.h"
 
+#include <QFuture>
+#include <QFutureWatcher>
+#include <QMetaEnum>
+#include <QSettings>
 #include <QStringList>
 #include <QUrl>
-#include <QMetaEnum>
+#include <QtConcurrentRun>
 
 #include <boost/bind.hpp>
 
+using smart_playlists::Generator;
+using smart_playlists::GeneratorMimeData;
+using smart_playlists::GeneratorPtr;
+using smart_playlists::QueryGenerator;
+
+const char* LibraryModel::kSmartPlaylistsMimeType = "application/x-clementine-smart-playlist-generator";
+const char* LibraryModel::kSmartPlaylistsSettingsGroup = "SerialisedSmartPlaylists";
+const int LibraryModel::kSmartPlaylistsVersion = 3;
+
+typedef QFuture<SqlRowList> RootQueryFuture;
+typedef QFutureWatcher<SqlRowList> RootQueryWatcher;
 
 LibraryModel::LibraryModel(LibraryBackend* backend, QObject* parent)
   : SimpleTreeModel<LibraryItem>(new LibraryItem(this), parent),
     backend_(backend),
     dir_model_(new LibraryDirectoryModel(backend, this)),
+    show_smart_playlists_(false),
+    show_various_artists_(true),
+    total_song_count_(0),
     artist_icon_(":/icons/22x22/x-clementine-artist.png"),
     album_icon_(":/icons/22x22/x-clementine-album.png"),
-    no_cover_icon_(":nocover.png")
+    no_cover_icon_(":nocover.png"),
+    playlists_dir_icon_(IconLoader::Load("folder-sound")),
+    playlist_icon_(":/icons/22x22/x-clementine-albums.png")
 {
   root_->lazy_loaded = true;
 
@@ -52,7 +76,9 @@ LibraryModel::~LibraryModel() {
 void LibraryModel::Init() {
   connect(backend_, SIGNAL(SongsDiscovered(SongList)), SLOT(SongsDiscovered(SongList)));
   connect(backend_, SIGNAL(SongsDeleted(SongList)), SLOT(SongsDeleted(SongList)));
-  connect(backend_, SIGNAL(TotalSongCountUpdated(int)), SIGNAL(TotalSongCountUpdated(int)));
+  connect(backend_, SIGNAL(SongsStatisticsChanged(SongList)), SLOT(SongsStatisticsChanged(SongList)));
+  connect(backend_, SIGNAL(DatabaseReset()), SLOT(Reset()));
+  connect(backend_, SIGNAL(TotalSongCountUpdated(int)), SLOT(TotalSongCountUpdatedSlot(int)));
 
   backend_->UpdateTotalSongCountAsync();
 
@@ -131,6 +157,17 @@ void LibraryModel::SongsDiscovered(const SongList& songs) {
   }
 }
 
+void LibraryModel::SongsStatisticsChanged(const SongList& songs) {
+  // This is called if there was a minor change to the songs that will not
+  // normally require the library to be restructured.  We can just update our
+  // internal cache of Song objects without worrying about resetting the model.
+  foreach (const Song& song, songs) {
+    if (song_nodes_.contains(song.id())) {
+      song_nodes_[song.id()]->metadata = song;
+    }
+  }
+}
+
 LibraryItem* LibraryModel::CreateCompilationArtistNode(bool signal, LibraryItem* parent) {
   if (signal)
     beginInsertRows(ItemToIndex(parent), parent->children.count(), parent->children.count());
@@ -314,7 +351,9 @@ QVariant LibraryModel::data(const LibraryItem* item, int role) const {
       return item->DisplayText();
 
     case Qt::DecorationRole:
-      switch (item->type)
+      switch (item->type) {
+        case LibraryItem::Type_PlaylistContainer:
+          return playlists_dir_icon_;
         case LibraryItem::Type_Container:
           switch (container_type) {
             case GroupBy_Album:
@@ -325,13 +364,20 @@ QVariant LibraryModel::data(const LibraryItem* item, int role) const {
             default:
               break;
           }
+          break;
+        case LibraryItem::Type_SmartPlaylist:
+          return playlist_icon_;
         default:
           break;
+      }
       break;
 
     case Role_Type:
       return item->type;
 
+    case Role_IsDivider:
+      return item->type == LibraryItem::Type_Divider;
+
     case Role_ContainerType:
       return container_type;
 
@@ -347,11 +393,42 @@ QVariant LibraryModel::data(const LibraryItem* item, int role) const {
   return QVariant();
 }
 
+SqlRowList LibraryModel::RunRootQuery(const QueryOptions& query_options,
+                                      const Grouping& group_by) {
+  // Warning: Some copy-paste with LazyPopulate here
+
+  // Information about what we want the children to be
+  GroupBy child_type = group_by[0];
+
+  // Initialise the query.  child_type says what type of thing we want (artists,
+  // songs, etc.)
+  LibraryQuery q(query_options);
+  InitQuery(child_type, &q);
+
+  // Top-level artists is special - we don't want compilation albums appearing
+  if (child_type == GroupBy_Artist) {
+    q.AddCompilationRequirement(false);
+  }
+
+  // Execute the query
+  QMutexLocker l(backend_->db()->Mutex());
+  if (!backend_->ExecQuery(&q))
+    return SqlRowList();
+
+  SqlRowList rows;
+  while (q.Next()) {
+    rows << SqlRow(q);
+  }
+  return rows;
+}
+
 void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
   if (parent->lazy_loaded)
     return;
   parent->lazy_loaded = true;
 
+  // Warning: Some copy-paste with RunRootQuery here
+
   // Information about what we want the children to be
   int child_level = parent->container_level + 1;
   GroupBy child_type = child_level >= 3 ? GroupBy_None : group_by_[child_level];
@@ -380,9 +457,12 @@ void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
 
   // Step through the results
   while (q.Next()) {
+    // Warning: Some copy-paste with ResetAsyncQueryFinished here
+
     // Create the item - it will get inserted into the model here
     LibraryItem* item =
-        ItemFromQuery(child_type, signal, child_level == 0, parent, q, child_level);
+        ItemFromQuery(child_type, signal, child_level == 0, parent, SqlRow(q),
+                      child_level);
 
     // Save a pointer to it for later
     if (child_type == GroupBy_None)
@@ -392,7 +472,43 @@ void LibraryModel::LazyPopulate(LibraryItem* parent, bool signal) {
   }
 }
 
-void LibraryModel::Reset() {
+void LibraryModel::ResetAsync() {
+  RootQueryFuture future = QtConcurrent::run(
+        this, &LibraryModel::RunRootQuery, query_options_, group_by_);
+  RootQueryWatcher* watcher = new RootQueryWatcher(this);
+  watcher->setFuture(future);
+
+  connect(watcher, SIGNAL(finished()), SLOT(ResetAsyncQueryFinished()));
+}
+
+void LibraryModel::ResetAsyncQueryFinished() {
+  RootQueryWatcher* watcher = static_cast<RootQueryWatcher*>(sender());
+  const SqlRowList rows = watcher->result();
+  watcher->deleteLater();
+
+  BeginReset();
+  root_->lazy_loaded = true;
+
+  foreach (const SqlRow& row, rows) {
+    // Warning: Some copy-paste with LazyPopulate here
+
+    const GroupBy child_type = group_by_[0];
+
+    // Create the item - it will get inserted into the model here
+    LibraryItem* item =
+        ItemFromQuery(child_type, false, true, root_, row, 0);
+
+    // Save a pointer to it for later
+    if (child_type == GroupBy_None)
+      song_nodes_[item->metadata.id()] = item;
+    else
+      container_nodes_[0][item->key] = item;
+  }
+
+  reset();
+}
+
+void LibraryModel::BeginReset() {
   delete root_;
   song_nodes_.clear();
   container_nodes_[0].clear();
@@ -400,14 +516,25 @@ void LibraryModel::Reset() {
   container_nodes_[2].clear();
   divider_nodes_.clear();
   compilation_artist_node_ = NULL;
+  smart_playlist_node_ = NULL;
 
   root_ = new LibraryItem(this);
   root_->lazy_loaded = false;
 
-  // Various artists?
-  if (group_by_[0] == GroupBy_Artist &&
-      backend_->HasCompilations(query_options_))
-    CreateCompilationArtistNode(false, root_);
+  if (show_various_artists_) {
+    // Various artists?
+    if (group_by_[0] == GroupBy_Artist &&
+        backend_->HasCompilations(query_options_))
+      CreateCompilationArtistNode(false, root_);
+  }
+
+  // Smart playlists?
+  if (show_smart_playlists_ && query_options_.filter.isEmpty())
+    CreateSmartPlaylists();
+}
+
+void LibraryModel::Reset() {
+  BeginReset();
 
   // Populate top level
   LazyPopulate(root_, false);
@@ -507,29 +634,29 @@ LibraryItem* LibraryModel::InitItem(GroupBy type, bool signal, LibraryItem *pare
 }
 
 LibraryItem* LibraryModel::ItemFromQuery(GroupBy type,
-                                    bool signal, bool create_divider,
-                                    LibraryItem* parent, const LibraryQuery& q,
-                                    int container_level) {
+                                         bool signal, bool create_divider,
+                                         LibraryItem* parent, const SqlRow& row,
+                                         int container_level) {
   LibraryItem* item = InitItem(type, signal, parent, container_level);
   int year = 0;
 
   switch (type) {
   case GroupBy_Artist:
-    item->key = q.Value(0).toString();
+    item->key = row.value(0).toString();
     item->display_text = TextOrUnknown(item->key);
     item->sort_text = SortTextForArtist(item->key);
     break;
 
   case GroupBy_YearAlbum:
-    year = qMax(0, q.Value(0).toInt());
-    item->metadata.set_year(q.Value(0).toInt());
-    item->metadata.set_album(q.Value(1).toString());
+    year = qMax(0, row.value(0).toInt());
+    item->metadata.set_year(row.value(0).toInt());
+    item->metadata.set_album(row.value(1).toString());
     item->key = PrettyYearAlbum(year, item->metadata.album());
     item->sort_text = SortTextForYear(year) + item->metadata.album();
     break;
 
   case GroupBy_Year:
-    year = qMax(0, q.Value(0).toInt());
+    year = qMax(0, row.value(0).toInt());
     item->key = QString::number(year);
     item->sort_text = SortTextForYear(year) + " ";
     break;
@@ -538,20 +665,20 @@ LibraryItem* LibraryModel::ItemFromQuery(GroupBy type,
   case GroupBy_Genre:
   case GroupBy_Album:
   case GroupBy_AlbumArtist:
-    item->key = q.Value(0).toString();
+    item->key = row.value(0).toString();
     item->display_text = TextOrUnknown(item->key);
     item->sort_text = SortTextForArtist(item->key);
     break;
 
   case GroupBy_FileType:
-    item->metadata.set_filetype(Song::FileType(q.Value(0).toInt()));
+    item->metadata.set_filetype(Song::FileType(row.value(0).toInt()));
     item->key = item->metadata.TextForFiletype();
     break;
 
   case GroupBy_None:
-    item->metadata.InitFromQuery(q);
+    item->metadata.InitFromQuery(row);
     item->key = item->metadata.title();
-    item->display_text = item->metadata.PrettyTitleWithArtist();
+    item->display_text = item->metadata.TitleWithCompilationArtist();
     item->sort_text = SortTextForSong(item->metadata);
     break;
   }
@@ -604,7 +731,7 @@ LibraryItem* LibraryModel::ItemFromSong(GroupBy type,
   case GroupBy_None:
     item->metadata = s;
     item->key = s.title();
-    item->display_text = s.PrettyTitleWithArtist();
+    item->display_text = s.TitleWithCompilationArtist();
     item->sort_text = SortTextForSong(s);
     break;
   }
@@ -696,6 +823,7 @@ Qt::ItemFlags LibraryModel::flags(const QModelIndex& index) const {
   switch (IndexToItem(index)->type) {
   case LibraryItem::Type_Song:
   case LibraryItem::Type_Container:
+  case LibraryItem::Type_SmartPlaylist:
     return Qt::ItemIsSelectable |
            Qt::ItemIsEnabled |
            Qt::ItemIsDragEnabled;
@@ -711,6 +839,20 @@ QStringList LibraryModel::mimeTypes() const {
 }
 
 QMimeData* LibraryModel::mimeData(const QModelIndexList& indexes) const {
+  if (indexes.isEmpty())
+    return NULL;
+
+  // Special case: a smart playlist was dragged
+  if (IndexToItem(indexes.first())->type == LibraryItem::Type_SmartPlaylist) {
+    GeneratorPtr generator = CreateGenerator(indexes.first());
+    if (!generator)
+      return NULL;
+
+    GeneratorMimeData* data = new GeneratorMimeData(generator);
+    data->setData(kSmartPlaylistsMimeType, QByteArray());
+    return data;
+  }
+
   SongMimeData* data = new SongMimeData;
   QList<QUrl> urls;
   QSet<int> song_ids;
@@ -780,12 +922,12 @@ SongList LibraryModel::GetChildSongs(const QModelIndex &index) const {
 
 void LibraryModel::SetFilterAge(int age) {
   query_options_.max_age = age;
-  Reset();
+  ResetAsync();
 }
 
 void LibraryModel::SetFilterText(const QString& text) {
   query_options_.filter = text;
-  Reset();
+  ResetAsync();
 }
 
 bool LibraryModel::canFetchMore(const QModelIndex &parent) const {
@@ -799,7 +941,7 @@ bool LibraryModel::canFetchMore(const QModelIndex &parent) const {
 void LibraryModel::SetGroupBy(const Grouping& g) {
   group_by_ = g;
 
-  Reset();
+  ResetAsync();
   emit GroupingChanged(g);
 }
 
@@ -822,3 +964,152 @@ LibraryModel::GroupBy& LibraryModel::Grouping::operator [](int i) {
   Q_ASSERT(0);
   return first;
 }
+
+void LibraryModel::CreateSmartPlaylists() {
+  smart_playlist_node_ = new LibraryItem(LibraryItem::Type_PlaylistContainer, root_);
+  smart_playlist_node_->container_level = 0;
+  smart_playlist_node_->sort_text = "\0";
+  smart_playlist_node_->key = tr("Smart playlists");
+
+  QSettings s;
+  s.beginGroup(kSmartPlaylistsSettingsGroup);
+  int version = s.value(backend_->songs_table() + "_version", 0).toInt();
+
+  // How many defaults do we have to write?
+  int unwritten_defaults = 0;
+  for (int i=version; i < default_smart_playlists_.count() ; ++i) {
+    unwritten_defaults += default_smart_playlists_[i].count();
+  }
+
+  // Save the defaults if there are any unwritten ones
+  if (unwritten_defaults) {
+    // How many items are stored already?
+    int playlist_index = s.beginReadArray(backend_->songs_table());
+    s.endArray();
+
+    // Append the new ones
+    s.beginWriteArray(backend_->songs_table(), playlist_index + unwritten_defaults);
+    for (; version < default_smart_playlists_.count() ; ++version) {
+      foreach (smart_playlists::GeneratorPtr gen, default_smart_playlists_[version]) {
+        SaveGenerator(&s, playlist_index++, gen);
+      }
+    }
+    s.endArray();
+  }
+
+  s.setValue(backend_->songs_table() + "_version", version);
+
+  const int count = s.beginReadArray(backend_->songs_table());
+  for (int i=0 ; i<count ; ++i) {
+    s.setArrayIndex(i);
+    ItemFromSmartPlaylist(s, false);
+  }
+}
+
+void LibraryModel::ItemFromSmartPlaylist(const QSettings& s, bool notify) const {
+  LibraryItem* item = new LibraryItem(LibraryItem::Type_SmartPlaylist,
+                                      notify ? NULL : smart_playlist_node_);
+  item->display_text = s.value("name").toString();
+  item->sort_text = item->display_text;
+  item->key = s.value("type").toString();
+  item->smart_playlist_data = s.value("data").toByteArray();
+  item->lazy_loaded = true;
+
+  if (notify)
+    item->InsertNotify(smart_playlist_node_);
+}
+
+void LibraryModel::AddGenerator(GeneratorPtr gen) {
+  QSettings s;
+  s.beginGroup(kSmartPlaylistsSettingsGroup);
+
+  // Count the existing items
+  const int count = s.beginReadArray(backend_->songs_table());
+  s.endArray();
+
+  // Add this one to the end
+  s.beginWriteArray(backend_->songs_table(), count + 1);
+  SaveGenerator(&s, count, gen);
+
+  // Add it to the model
+  ItemFromSmartPlaylist(s, true);
+
+  s.endArray();
+}
+
+void LibraryModel::UpdateGenerator(const QModelIndex& index, GeneratorPtr gen) {
+  if (index.parent() != ItemToIndex(smart_playlist_node_))
+    return;
+  LibraryItem* item = IndexToItem(index);
+  if (!item)
+    return;
+
+  // Update the config
+  QSettings s;
+  s.beginGroup(kSmartPlaylistsSettingsGroup);
+
+  // Count the existing items
+  const int count = s.beginReadArray(backend_->songs_table());
+  s.endArray();
+
+  s.beginWriteArray(backend_->songs_table(), count);
+  SaveGenerator(&s, index.row(), gen);
+
+  // Update the text of the item
+  item->display_text = gen->name();
+  item->sort_text = item->display_text;
+  item->key = gen->type();
+  item->smart_playlist_data = gen->Save();
+  item->ChangedNotify();
+}
+
+void LibraryModel::DeleteGenerator(const QModelIndex& index) {
+  if (index.parent() != ItemToIndex(smart_playlist_node_))
+    return;
+
+  // Remove the item from the tree
+  smart_playlist_node_->DeleteNotify(index.row());
+
+  QSettings s;
+  s.beginGroup(kSmartPlaylistsSettingsGroup);
+
+  // Rewrite all the items to the settings
+  s.beginWriteArray(backend_->songs_table(), smart_playlist_node_->children.count());
+  int i = 0;
+  foreach (LibraryItem* item, smart_playlist_node_->children) {
+    s.setArrayIndex(i++);
+    s.setValue("name", item->display_text);
+    s.setValue("type", item->key);
+    s.setValue("data", item->smart_playlist_data);
+  }
+  s.endArray();
+}
+
+void LibraryModel::SaveGenerator(QSettings* s, int i, GeneratorPtr generator) const {
+  s->setArrayIndex(i);
+  s->setValue("name", generator->name());
+  s->setValue("type", generator->type());
+  s->setValue("data", generator->Save());
+}
+
+GeneratorPtr LibraryModel::CreateGenerator(const QModelIndex& index) const {
+  GeneratorPtr ret;
+
+  const LibraryItem* item = IndexToItem(index);
+  if (!item || item->type != LibraryItem::Type_SmartPlaylist)
+    return ret;
+
+  ret = Generator::Create(item->key);
+  if (!ret)
+    return ret;
+
+  ret->set_name(item->display_text);
+  ret->set_library(backend());
+  ret->Load(item->smart_playlist_data);
+  return ret;
+}
+
+void LibraryModel::TotalSongCountUpdatedSlot(int count) {
+  total_song_count_ = count;
+  emit TotalSongCountUpdated(count);
+}
