@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,6 +19,7 @@
 #define LIBRARYVIEW_H
 
 #include "core/song.h"
+#include "ui/edittagdialog.h"
 #include "widgets/autoexpandingtreeview.h"
 
 #include <QStyledItemDelegate>
@@ -29,6 +31,8 @@ class LibraryModel;
 class OrganiseDialog;
 class TaskManager;
 
+namespace smart_playlists { class Wizard; }
+
 class LibraryItemDelegate : public QStyledItemDelegate {
  public:
   LibraryItemDelegate(QObject* parent);
@@ -70,7 +74,7 @@ class LibraryView : public AutoExpandingTreeView {
   void paintEvent(QPaintEvent* event);
   void mouseReleaseEvent(QMouseEvent* e);
   void contextMenuEvent(QContextMenuEvent* e);
-  void keyReleaseEvent(QKeyEvent* e);
+  void keyPressEvent(QKeyEvent* e);
 
  private slots:
   void Load();
@@ -78,9 +82,17 @@ class LibraryView : public AutoExpandingTreeView {
   void Organise();
   void CopyToDevice();
   void Delete();
+  void EditTracks();
   void ShowInVarious();
   void NoShowInVarious();
 
+  void NewSmartPlaylist();
+  void EditSmartPlaylist();
+  void DeleteSmartPlaylist();
+
+  void NewSmartPlaylistFinished();
+  void EditSmartPlaylistFinished();
+
   void DeleteFinished(const SongList& songs_with_errors);
 
  private:
@@ -104,10 +116,16 @@ class LibraryView : public AutoExpandingTreeView {
   QAction* organise_;
   QAction* copy_to_device_;
   QAction* delete_;
+  QAction* edit_track_;
   QAction* show_in_various_;
   QAction* no_show_in_various_;
 
+  QAction* new_smart_playlist_;
+  QAction* edit_smart_playlist_;
+  QAction* delete_smart_playlist_;
+
   boost::scoped_ptr<OrganiseDialog> organise_dialog_;
+  boost::scoped_ptr<EditTagDialog> edit_tag_dialog_;
 
   bool is_in_keyboard_search_;
 };
