@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,15 +23,18 @@
 
 #include "config.h"
 #include "core/commandlineoptions.h"
+#include "core/database.h"
 #include "core/encoding.h"
 #include "core/mac_startup.h"
-#include "core/networkaccessmanager.h"
+#include "core/network.h"
 #include "core/player.h"
 #include "core/potranslator.h"
 #include "core/song.h"
+#include "core/utilities.h"
 #include "engines/enginebase.h"
 #include "library/directory.h"
 #include "radio/lastfmservice.h"
+#include "smartplaylists/generator.h"
 #include "ui/equalizer.h"
 #include "ui/iconloader.h"
 #include "ui/mainwindow.h"
@@ -38,20 +42,19 @@
 #include "qtsingleapplication.h"
 #include "qtsinglecoreapplication.h"
 
-#include <QtDebug>
-#include <QLibraryInfo>
-#include <QTranslator>
 #include <QDir>
+#include <QLibraryInfo>
+#include <QSqlDatabase>
+#include <QSqlQuery>
+#include <QTranslator>
+#include <QtDebug>
 
 #include <glib-object.h>
 #include <glib/gutils.h>
 
-#ifdef Q_WS_X11
-#  include <QDBusConnection>
-#  include <QDBusMetaType>
-#  include "core/mpris.h"
-#  include "widgets/osd.h"
-#endif
+#include <echonest/Config.h>
+
+
 
 #ifdef HAVE_GSTREAMER
 #  include <gst/gst.h>
@@ -104,6 +107,23 @@ int main(int argc, char *argv[]) {
   QCoreApplication::setOrganizationName("Clementine");
   QCoreApplication::setOrganizationDomain("davidsansome.com");
 
+#ifdef Q_OS_DARWIN
+  // Must happen after QCoreApplication::setOrganizationName().
+  if (mac::MigrateLegacyConfigFiles()) {
+    QSqlDatabase db = QSqlDatabase::addDatabase("QSQLITE");
+    db.setDatabaseName(Utilities::GetConfigPath(
+        Utilities::Path_Root) + "/" + Database::kDatabaseFilename);
+    db.open();
+    QSqlQuery query(
+        "UPDATE songs SET art_manual = replace("
+        "art_manual, '.config', 'Library/Application Support') "
+        "WHERE art_manual LIKE '%.config%'", db);
+    query.exec();
+    db.close();
+    QSqlDatabase::removeDatabase(db.connectionName());
+  }
+#endif
+
   // This makes us show up nicely in gnome-volume-control
   g_type_init();
   g_set_application_name(QCoreApplication::applicationName().toLocal8Bit());
@@ -122,6 +142,8 @@ int main(int argc, char *argv[]) {
   qRegisterMetaTypeStreamOperators<Equalizer::Params>("Equalizer::Params");
   qRegisterMetaType<const char*>("const char*");
   qRegisterMetaType<QNetworkReply*>("QNetworkReply*");
+  qRegisterMetaType<QNetworkReply**>("QNetworkReply**");
+  qRegisterMetaType<smart_playlists::GeneratorPtr>("smart_playlists::GeneratorPtr");
 
 #ifdef HAVE_GSTREAMER
   qRegisterMetaType<GstBuffer*>("GstBuffer*");
@@ -195,22 +217,14 @@ int main(int argc, char *argv[]) {
   // Icons
   IconLoader::Init();
 
-  NetworkAccessManager network;
-
-  // MPRIS DBus interface.
-#ifdef Q_WS_X11
-  qDBusRegisterMetaType<DBusStatus>();
-  qDBusRegisterMetaType<Version>();
-  qDBusRegisterMetaType<QImage>();
-  QDBusConnection::sessionBus().registerService("org.mpris.clementine");
-  MPRIS mpris;
-#endif
+  Echonest::Config::instance()->setAPIKey("DFLFLJBUF4EGTXHIG");
+  Echonest::Config::instance()->setNetworkAccessManager(new NetworkAccessManager);
 
   // Seed the random number generator
   srand(time(NULL));
 
   // Window
-  MainWindow w(&network, options.engine());
+  MainWindow w(options.engine());
 
   QObject::connect(&a, SIGNAL(messageReceived(QByteArray)), &w, SLOT(CommandlineOptionsReceived(QByteArray)));
   w.CommandlineOptionsReceived(options);
