@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,15 +26,22 @@
 #include "core/modelfuturewatcher.h"
 #include "library/library.h"
 #include "library/librarybackend.h"
+#include "library/librarymodel.h"
 #include "library/libraryplaylistitem.h"
-#include "radio/magnatuneservice.h"
+#include "radio/jamendoplaylistitem.h"
+#include "radio/jamendoservice.h"
 #include "radio/magnatuneplaylistitem.h"
+#include "radio/magnatuneservice.h"
 #include "radio/radiomimedata.h"
 #include "radio/radiomodel.h"
 #include "radio/radioplaylistitem.h"
 #include "radio/savedradio.h"
+#include "smartplaylists/generator.h"
+#include "smartplaylists/generatorinserter.h"
+#include "smartplaylists/generatormimedata.h"
 
 #include <QtDebug>
+#include <QApplication>
 #include <QMimeData>
 #include <QBuffer>
 #include <QFileInfo>
@@ -46,6 +54,10 @@
 
 #include <lastfm/ScrobblePoint>
 
+using smart_playlists::Generator;
+using smart_playlists::GeneratorInserter;
+using smart_playlists::GeneratorPtr;
+
 using boost::shared_ptr;
 
 const char* Playlist::kRowsMimetype = "application/x-clementine-playlist-rows";
@@ -57,6 +69,7 @@ Playlist::Playlist(PlaylistBackend* backend,
                    int id,
                    QObject *parent)
   : QAbstractListModel(parent),
+    is_loading_(false),
     proxy_(new PlaylistFilter(this)),
     queue_(new Queue(this)),
     backend_(backend),
@@ -95,6 +108,15 @@ Playlist::~Playlist() {
   library_items_by_id_.clear();
 }
 
+template<typename T>
+QModelIndex InsertSongItems(Playlist* playlist, const SongList& songs, int pos) {
+  PlaylistItemList items;
+  foreach (const Song& song, songs) {
+    items << PlaylistItemPtr(new T(song));
+  }
+  return playlist->InsertItems(items, pos);
+}
+
 QVariant Playlist::headerData(int section, Qt::Orientation, int role) const {
   if (role != Qt::DisplayRole && role != Qt::ToolTipRole)
     return QVariant();
@@ -117,6 +139,8 @@ bool Playlist::column_is_editable(Playlist::Column column) {
     case Column_Disc:
     case Column_Year:
     case Column_Genre:
+    case Column_Score:
+    case Column_Comment:
       return true;
     default:
       break;
@@ -158,6 +182,12 @@ bool Playlist::set_column_value(Song& song, Playlist::Column column,
     case Column_Genre:
       song.set_genre(value.toString());
       break;
+    case Column_Score:
+      song.set_score(value.toInt());
+      break;
+    case Column_Comment:
+      song.set_comment(value.toString());
+      break;
     default:
       break;
   }
@@ -178,10 +208,14 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
     case Role_QueuePosition:
       return queue_->PositionOf(index);
 
+    case Role_CanSetRating:
+      return index.column() == Column_Rating &&
+             items_[index.row()]->IsLocalLibraryItem();
+
     case Qt::EditRole:
     case Qt::ToolTipRole:
     case Qt::DisplayRole: {
-      shared_ptr<PlaylistItem> item = items_[index.row()];
+      PlaylistItemPtr item = items_[index.row()];
       Song song = item->Metadata();
 
       // Don't forget to change Playlist::CompareItems when adding new columns
@@ -192,25 +226,36 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
           if (!song.basefilename().isEmpty())
             return song.basefilename();
           return song.filename();
-        case Column_Artist:   return song.artist();
-        case Column_Album:    return song.album();
-        case Column_Length:   return song.length();
-        case Column_Track:    return song.track();
-        case Column_Disc:     return song.disc();
-        case Column_Year:     return song.year();
-        case Column_Genre:    return song.genre();
-        case Column_AlbumArtist: return song.albumartist();
-        case Column_Composer: return song.composer();
+        case Column_Artist:       return song.artist();
+        case Column_Album:        return song.album();
+        case Column_Length:       return song.length();
+        case Column_Track:        return song.track();
+        case Column_Disc:         return song.disc();
+        case Column_Year:         return song.year();
+        case Column_Genre:        return song.genre();
+        case Column_AlbumArtist:  return song.albumartist();
+        case Column_Composer:     return song.composer();
 
-        case Column_BPM:      return song.bpm();
-        case Column_Bitrate:  return song.bitrate();
-        case Column_Samplerate: return song.samplerate();
-        case Column_Filename: return song.filename();
+        case Column_Rating:       return song.rating();
+        case Column_PlayCount:    return song.playcount();
+        case Column_SkipCount:    return song.skipcount();
+        case Column_LastPlayed:   return song.lastplayed();
+        case Column_Score:        return song.score();
+
+        case Column_BPM:          return song.bpm();
+        case Column_Bitrate:      return song.bitrate();
+        case Column_Samplerate:   return song.samplerate();
+        case Column_Filename:     return song.filename();
         case Column_BaseFilename: return song.basefilename();
-        case Column_Filesize: return song.filesize();
-        case Column_Filetype: return song.filetype();
+        case Column_Filesize:     return song.filesize();
+        case Column_Filetype:     return song.filetype();
         case Column_DateModified: return song.mtime();
-        case Column_DateCreated: return song.ctime();
+        case Column_DateCreated:  return song.ctime();
+
+        case Column_Comment:
+          if (role == Qt::DisplayRole)
+            return song.comment().simplified();
+          return song.comment();
       }
     }
 
@@ -224,12 +269,20 @@ QVariant Playlist::data(const QModelIndex& index, int role) const {
         case Column_Bitrate:
         case Column_Samplerate:
         case Column_Filesize:
+        case Column_PlayCount:
+        case Column_SkipCount:
           return QVariant(Qt::AlignRight | Qt::AlignVCenter);
-
+        case Column_Score:
+          return QVariant(Qt::AlignCenter | Qt::AlignVCenter);
         default:
           return QVariant(Qt::AlignLeft | Qt::AlignVCenter);
       }
 
+    case Qt::ForegroundRole:
+      if (items_[index.row()]->IsDynamicHistory())
+        return QApplication::palette().brush(QPalette::Disabled, QPalette::Text);
+      return QVariant();
+
     default:
       return QVariant();
   }
@@ -245,10 +298,16 @@ bool Playlist::setData(const QModelIndex &index, const QVariant &value, int) {
   if(!set_column_value(song, (Column)index.column(), value))
     return false;
 
-  QFuture<bool> future = song.BackgroundSave();
-  ModelFutureWatcher<bool>* watcher = new ModelFutureWatcher<bool>(index, this);
-  watcher->setFuture(future);
-  connect(watcher, SIGNAL(finished()), SLOT(SongSaveComplete()));
+  if((Column)index.column() == Column_Score) {
+    // The score is only saved in the database, not the file
+    library_->AddOrUpdateSongs(SongList() << song);
+    emit EditingFinished(index);
+  } else {
+    QFuture<bool> future = song.BackgroundSave();
+    ModelFutureWatcher<bool>* watcher = new ModelFutureWatcher<bool>(index, this);
+    watcher->setFuture(future);
+    connect(watcher, SIGNAL(finished()), SLOT(SongSaveComplete()));
+  }
   return true;
 }
 
@@ -261,8 +320,6 @@ void Playlist::SongSaveComplete() {
     ModelFutureWatcher<void>* watcher = new ModelFutureWatcher<void>(index, this);
     watcher->setFuture(future);
     connect(watcher, SIGNAL(finished()), SLOT(ItemReloadComplete()));
-
-    emit EditingFinished(index);
   }
 }
 
@@ -272,6 +329,7 @@ void Playlist::ItemReloadComplete() {
   const QPersistentModelIndex& index = watcher->index();
   if (index.isValid()) {
     emit dataChanged(index, index);
+    emit EditingFinished(index);
   }
 }
 
@@ -440,6 +498,13 @@ void Playlist::set_current_index(int i) {
 
   current_item_index_ = QPersistentModelIndex(index(i, 0, QModelIndex()));
 
+  if (current_item_index_.row() == queue_->PeekNext()) {
+    queue_->TakeNext();
+  }
+
+  if (current_item_index_ == old_current)
+    return;
+
   if (current_item_index_.isValid()) {
     last_played_item_index_ = current_item_index_;
     current_item_ = items_[current_item_index_.row()];
@@ -448,16 +513,17 @@ void Playlist::set_current_index(int i) {
     current_item_.reset();
   }
 
-  if (old_current.isValid())
-    emit dataChanged(old_current, old_current.sibling(old_current.row(), ColumnCount-1));
+  if (old_current.isValid()) {
+    if (dynamic_playlist_) {
+      items_[old_current.row()]->SetDynamicHistory(true);
+    }
 
-  if (current_item_index_.isValid() && current_item_index_ != old_current) {
-    emit dataChanged(current_item_index_, current_item_index_.sibling(current_item_index_.row(), ColumnCount-1));
-    emit CurrentSongChanged(current_item_metadata());
+    emit dataChanged(old_current, old_current.sibling(old_current.row(), ColumnCount-1));
   }
 
-  if (current_item_index_.row() == queue_->PeekNext()) {
-    queue_->TakeNext();
+  if (current_item_index_.isValid()) {
+    emit dataChanged(current_item_index_, current_item_index_.sibling(current_item_index_.row(), ColumnCount-1));
+    emit CurrentSongChanged(current_item_metadata());
   }
 
   // Update the virtual index
@@ -477,11 +543,29 @@ void Playlist::set_current_index(int i) {
   else
     current_virtual_index_ = i;
 
+  if (dynamic_playlist_ && current_item_index_.isValid() && old_current.isValid()) {
+    using smart_playlists::Generator;
+
+    // Add more dynamic playlist items
+    const int count = current_item_index_.row() + Generator::kDynamicFuture - items_.count();
+    if (count > 0) {
+      GeneratorInserter* inserter = new GeneratorInserter(task_manager_, library_, this);
+      connect(inserter, SIGNAL(Error(QString)), SIGNAL(LoadTracksError(QString)));
+      connect(inserter, SIGNAL(PlayRequested(QModelIndex)), SIGNAL(PlayRequested(QModelIndex)));
+
+      inserter->Load(this, -1, false, dynamic_playlist_, count);
+    }
+
+    // Remove the first item
+    if (current_item_index_.row() > Generator::kDynamicHistory) {
+      RemoveItemsWithoutUndo(0, 1);
+    }
+  }
+
   UpdateScrobblePoint();
 }
 
 Qt::ItemFlags Playlist::flags(const QModelIndex &index) const {
-
   Qt::ItemFlags flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable;
 
   if(column_is_editable((Column)index.column()))
@@ -494,7 +578,8 @@ Qt::ItemFlags Playlist::flags(const QModelIndex &index) const {
 }
 
 QStringList Playlist::mimeTypes() const {
-  return QStringList() << "text/uri-list" << kRowsMimetype;
+  return QStringList() << "text/uri-list" << kRowsMimetype
+                       << LibraryModel::kSmartPlaylistsMimeType;
 }
 
 Qt::DropActions Playlist::supportedDropActions() const {
@@ -505,19 +590,25 @@ bool Playlist::dropMimeData(const QMimeData* data, Qt::DropAction action, int ro
   if (action == Qt::IgnoreAction)
     return false;
 
+  using smart_playlists::GeneratorMimeData;
+
   if (const SongMimeData* song_data = qobject_cast<const SongMimeData*>(data)) {
     // Dragged from a library
     // We want to check if these songs are from the actual local file backend,
     // if they are we treat them differently.
-    if (song_data->backend->songs_table() == Library::kSongsTable)
-      InsertLibraryItems(song_data->songs, row);
-    else if (song_data->backend->songs_table() == MagnatuneService::kSongsTable)
-      InsertMagnatuneItems(song_data->songs, row);
+    if (song_data->backend && song_data->backend->songs_table() == Library::kSongsTable)
+      InsertSongItems<LibraryPlaylistItem>(this, song_data->songs, row);
+    else if (song_data->backend && song_data->backend->songs_table() == MagnatuneService::kSongsTable)
+      InsertSongItems<MagnatunePlaylistItem>(this, song_data->songs, row);
+    else if (song_data->backend && song_data->backend->songs_table() == JamendoService::kSongsTable)
+      InsertSongItems<JamendoPlaylistItem>(this, song_data->songs, row);
     else
-      InsertSongs(song_data->songs, row);
+      InsertSongItems<SongPlaylistItem>(this, song_data->songs, row);
   } else if (const RadioMimeData* radio_data = qobject_cast<const RadioMimeData*>(data)) {
     // Dragged from the Radio pane
     InsertRadioStations(radio_data->items, row, data->hasFormat(kPlayNowMimetype));
+  } else if (const GeneratorMimeData* generator_data = qobject_cast<const GeneratorMimeData*>(data)) {
+    InsertSmartPlaylist(generator_data->generator_, row, data->hasFormat(kPlayNowMimetype));
   } else if (data->hasFormat(kRowsMimetype)) {
     // Dragged from the playlist
     // Rearranging it is tricky...
@@ -565,6 +656,26 @@ void Playlist::InsertUrls(const QList<QUrl> &urls, bool play_now, int pos) {
   inserter->Load(this, pos, play_now, urls);
 }
 
+void Playlist::InsertSmartPlaylist(GeneratorPtr generator, int pos, bool play_now) {
+  GeneratorInserter* inserter = new GeneratorInserter(task_manager_, library_, this);
+  connect(inserter, SIGNAL(Error(QString)), SIGNAL(LoadTracksError(QString)));
+  connect(inserter, SIGNAL(PlayRequested(QModelIndex)), SIGNAL(PlayRequested(QModelIndex)));
+
+  inserter->Load(this, pos, play_now, generator);
+
+  if (generator->is_dynamic()) {
+    TurnOnDynamicPlaylist(generator);
+  }
+}
+
+void Playlist::TurnOnDynamicPlaylist(GeneratorPtr gen) {
+  dynamic_playlist_ = gen;
+  playlist_sequence_->SetUsingDynamicPlaylist(true);
+  ShuffleModeChanged(PlaylistSequence::Shuffle_Off);
+  emit DynamicModeChanged(true);
+  Save();
+}
+
 void Playlist::MoveItemsWithoutUndo(const QList<int> &source_rows, int pos) {
   layoutAboutToBeChanged();
   PlaylistItemList moved_items;
@@ -582,6 +693,7 @@ void Playlist::MoveItemsWithoutUndo(const QList<int> &source_rows, int pos) {
   // Put the items back in
   const int start = pos == -1 ? items_.count() : pos;
   for (int i=start ; i<start+moved_items.count() ; ++i) {
+    moved_items[i - start]->SetDynamicHistory(false);
     items_.insert(i, moved_items[i - start]);
   }
 
@@ -673,7 +785,7 @@ QModelIndex Playlist::InsertItemsWithoutUndo(const PlaylistItemList& items,
 
   beginInsertRows(QModelIndex(), start, end);
   for (int i=start ; i<=end ; ++i) {
-    boost::shared_ptr<PlaylistItem> item = items[i - start];
+    PlaylistItemPtr item = items[i - start];
     items_.insert(i, item);
     virtual_items_ << virtual_items_.count();
 
@@ -699,25 +811,20 @@ QModelIndex Playlist::InsertItemsWithoutUndo(const PlaylistItemList& items,
 }
 
 QModelIndex Playlist::InsertLibraryItems(const SongList& songs, int pos) {
-  PlaylistItemList items;
-  foreach (const Song& song, songs) {
-    items << shared_ptr<PlaylistItem>(new LibraryPlaylistItem(song));
-  }
-  return InsertItems(items, pos);
-}
-
-QModelIndex Playlist::InsertMagnatuneItems(const SongList& songs, int pos) {
-  PlaylistItemList items;
-  foreach (const Song& song, songs) {
-    items << shared_ptr<PlaylistItem>(new MagnatunePlaylistItem(song));
-  }
-  return InsertItems(items, pos);
+  return InsertSongItems<LibraryPlaylistItem>(this, songs, pos);
 }
 
 QModelIndex Playlist::InsertSongs(const SongList& songs, int pos) {
+  return InsertSongItems<SongPlaylistItem>(this, songs, pos);
+}
+
+QModelIndex Playlist::InsertSongsOrLibraryItems(const SongList& songs, int pos) {
   PlaylistItemList items;
   foreach (const Song& song, songs) {
-    items << shared_ptr<PlaylistItem>(new SongPlaylistItem(song));
+    if (song.id() == -1)
+      items << PlaylistItemPtr(new SongPlaylistItem(song));
+    else
+      items << PlaylistItemPtr(new LibraryPlaylistItem(song));
   }
   return InsertItems(items, pos);
 }
@@ -792,6 +899,12 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order,
     case Column_AlbumArtist:  strcmp(albumartist);
     case Column_Composer:     strcmp(composer);
 
+    case Column_Rating:       cmp(rating);
+    case Column_PlayCount:    cmp(playcount);
+    case Column_SkipCount:    cmp(skipcount);
+    case Column_LastPlayed:   cmp(lastplayed);
+    case Column_Score:        cmp(score);
+
     case Column_BPM:          cmp(bpm);
     case Column_Bitrate:      cmp(bitrate);
     case Column_Samplerate:   cmp(samplerate);
@@ -801,6 +914,8 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order,
     case Column_Filetype:     cmp(filetype);
     case Column_DateModified: cmp(mtime);
     case Column_DateCreated:  cmp(ctime);
+
+    case Column_Comment:      strcmp(comment);
   }
 
 #undef cmp
@@ -811,26 +926,34 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order,
 
 QString Playlist::column_name(Column column) {
   switch (column) {
-    case Column_Title:    return tr("Title");
-    case Column_Artist:   return tr("Artist");
-    case Column_Album:    return tr("Album");
-    case Column_Length:   return tr("Length");
-    case Column_Track:    return tr("Track");
-    case Column_Disc:     return tr("Disc");
-    case Column_Year:     return tr("Year");
-    case Column_Genre:    return tr("Genre");
-    case Column_AlbumArtist: return tr("Album artist");
-    case Column_Composer: return tr("Composer");
+    case Column_Title:        return tr("Title");
+    case Column_Artist:       return tr("Artist");
+    case Column_Album:        return tr("Album");
+    case Column_Length:       return tr("Length");
+    case Column_Track:        return tr("Track");
+    case Column_Disc:         return tr("Disc");
+    case Column_Year:         return tr("Year");
+    case Column_Genre:        return tr("Genre");
+    case Column_AlbumArtist:  return tr("Album artist");
+    case Column_Composer:     return tr("Composer");
 
-    case Column_BPM:      return tr("BPM");
-    case Column_Bitrate:  return tr("Bit rate");
-    case Column_Samplerate: return tr("Sample rate");
-    case Column_Filename: return tr("File name");
+    case Column_Rating:       return tr("Rating");
+    case Column_PlayCount:    return tr("Play count");
+    case Column_SkipCount:    return tr("Skip count");
+    case Column_LastPlayed:   return tr("Last played");
+    case Column_Score:        return tr("Score");
+
+    case Column_BPM:          return tr("BPM");
+    case Column_Bitrate:      return tr("Bit rate");
+    case Column_Samplerate:   return tr("Sample rate");
+    case Column_Filename:     return tr("File name");
     case Column_BaseFilename: return tr("File name (without path)");
-    case Column_Filesize: return tr("File size");
-    case Column_Filetype: return tr("File type");
+    case Column_Filesize:     return tr("File size");
+    case Column_Filetype:     return tr("File type");
     case Column_DateModified: return tr("Date modified");
-    case Column_DateCreated: return tr("Date created");
+    case Column_DateCreated:  return tr("Date created");
+
+    case Column_Comment:      return tr("Comment");
     default: return QString();
   }
   return "";
@@ -866,6 +989,7 @@ void Playlist::sort(int column, Qt::SortOrder order) {
   // TODO
   undo_stack_->clear();
 
+  emit PlaylistChanged();
   Save();
 }
 
@@ -893,10 +1017,10 @@ void Playlist::SetCurrentIsPaused(bool paused) {
 }
 
 void Playlist::Save() const {
-  if (!backend_)
+  if (!backend_ || is_loading_)
     return;
 
-  backend_->SavePlaylistAsync(id_, items_, last_played_index());
+  backend_->SavePlaylistAsync(id_, items_, last_played_index(), dynamic_playlist_);
 }
 
 namespace {
@@ -922,12 +1046,34 @@ void Playlist::ItemsLoaded() {
   watcher->deleteLater();
 
   PlaylistItemList items = watcher->future().results();
+  is_loading_ = true;
   InsertItems(items, 0);
+  is_loading_ = false;
 
   PlaylistBackend::Playlist p = backend_->GetPlaylist(id_);
 
   last_played_item_index_ =
       p.last_played == -1 ? QModelIndex() : index(p.last_played);
+
+  if (!p.dynamic_type.isEmpty()) {
+    GeneratorPtr gen = Generator::Create(p.dynamic_type);
+    if (gen) {
+      // Hack: can't think of a better way to get the right backend
+      LibraryBackend* backend = NULL;
+      if (p.dynamic_backend == library_->songs_table())
+        backend = library_;
+      else if (p.dynamic_backend == MagnatuneService::kSongsTable)
+        backend = RadioModel::Service<MagnatuneService>()->library_backend();
+      else if (p.dynamic_backend == JamendoService::kSongsTable)
+        backend = RadioModel::Service<JamendoService>()->library_backend();
+
+      if (backend) {
+        gen->set_library(backend);
+        gen->Load(p.dynamic_data);
+        TurnOnDynamicPlaylist(gen);
+      }
+    }
+  }
 }
 
 bool Playlist::removeRows(int row, int count, const QModelIndex& parent) {
@@ -948,7 +1094,7 @@ PlaylistItemList Playlist::RemoveItemsWithoutUndo(int row, int count) {
   // Remove items
   PlaylistItemList ret;
   for (int i=0 ; i<count ; ++i) {
-    boost::shared_ptr<PlaylistItem> item(items_.takeAt(row));
+    PlaylistItemPtr item(items_.takeAt(row));
     ret << item;
 
     if (item->type() == "Library") {
@@ -1053,10 +1199,63 @@ void Playlist::UpdateScrobblePoint() {
 
 void Playlist::Clear() {
   undo_stack_->push(new PlaylistUndoCommands::RemoveItems(this, 0, items_.count()));
+  TurnOffDynamicPlaylist();
 
   Save();
 }
 
+void Playlist::TurnOffDynamicPlaylist() {
+  dynamic_playlist_.reset();
+
+  if (playlist_sequence_) {
+    playlist_sequence_->SetUsingDynamicPlaylist(false);
+    ShuffleModeChanged(playlist_sequence_->shuffle_mode());
+  }
+  emit DynamicModeChanged(false);
+  Save();
+}
+
+void Playlist::RepopulateDynamicPlaylist() {
+  if (!dynamic_playlist_)
+    return;
+
+  RemoveItemsNotInQueue();
+  InsertSmartPlaylist(dynamic_playlist_);
+}
+
+void Playlist::RemoveItemsNotInQueue() {
+  if (queue_->is_empty()) {
+    RemoveItemsWithoutUndo(0, items_.count());
+    return;
+  }
+
+  int start = 0;
+  forever {
+    // Find a place to start - first row that isn't in the queue
+    forever {
+      if (start >= rowCount())
+        return;
+      if (!queue_->ContainsSourceRow(start))
+        break;
+      start ++;
+    }
+
+    // Figure out how many rows to remove - keep going until we find a row
+    // that is in the queue
+    int count = 1;
+    forever {
+      if (start + count >= rowCount())
+        break;
+      if (queue_->ContainsSourceRow(start + count))
+        break;
+      count ++;
+    }
+
+    RemoveItemsWithoutUndo(start, count);
+    start ++;
+  }
+}
+
 void Playlist::ReloadItems(const QList<int>& rows) {
   foreach (int row, rows) {
     item_at(row)->Reload();
@@ -1087,6 +1286,7 @@ void Playlist::Shuffle() {
   // TODO
   undo_stack_->clear();
 
+  emit PlaylistChanged();
   Save();
 }
 
@@ -1118,7 +1318,7 @@ QSortFilterProxyModel* Playlist::proxy() const {
 
 SongList Playlist::GetAllSongs() const {
   SongList ret;
-  foreach (boost::shared_ptr<PlaylistItem> item, items_) {
+  foreach (PlaylistItemPtr item, items_) {
     ret << item->Metadata();
   }
   return ret;
@@ -1126,7 +1326,7 @@ SongList Playlist::GetAllSongs() const {
 
 quint64 Playlist::GetTotalLength() const {
   quint64 ret = 0;
-  foreach (boost::shared_ptr<PlaylistItem> item, items_) {
+  foreach (PlaylistItemPtr item, items_) {
     int length = item->Metadata().length();
     if (length > 0)
       ret += length;
@@ -1173,3 +1373,12 @@ QDataStream& operator >>(QDataStream& s, Playlist*& p) {
   s.readRawData(reinterpret_cast<char*>(&p), sizeof(p));
   return s;
 }
+
+void Playlist::ItemChanged(PlaylistItemPtr item) {
+  for (int row=0 ; row<items_.count() ; ++row) {
+    if (items_[row] == item) {
+      emit dataChanged(index(row, 0), index(row, ColumnCount-1));
+      return;
+    }
+  }
+}
