@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,6 +27,7 @@
 #include "playlistsequence.h"
 #include "core/song.h"
 #include "radio/radioitem.h"
+#include "smartplaylists/generator_fwd.h"
 
 class LibraryBackend;
 class PlaylistBackend;
@@ -58,6 +60,7 @@ class Playlist : public QAbstractListModel {
            QObject* parent = 0);
   ~Playlist();
 
+  // Always add new columns to the end of this enum - the values are persisted
   enum Column {
     Column_Title = 0,
     Column_Artist,
@@ -80,6 +83,14 @@ class Playlist : public QAbstractListModel {
     Column_DateCreated,
     Column_DateModified,
 
+    Column_Rating,
+    Column_PlayCount,
+    Column_SkipCount,
+    Column_LastPlayed,
+    Column_Score,
+
+    Column_Comment,
+
     ColumnCount
   };
 
@@ -88,14 +99,14 @@ class Playlist : public QAbstractListModel {
     Role_IsPaused,
     Role_StopAfter,
     Role_QueuePosition,
+    Role_CanSetRating,
   };
 
   static const char* kRowsMimetype;
   static const char* kPlayNowMimetype;
 
   static bool CompareItems(int column, Qt::SortOrder order,
-                           boost::shared_ptr<PlaylistItem> a,
-                           boost::shared_ptr<PlaylistItem> b);
+                           PlaylistItemPtr a, PlaylistItemPtr b);
 
   static QString column_name(Column column);
   static bool column_is_editable(Playlist::Column column);
@@ -115,9 +126,10 @@ class Playlist : public QAbstractListModel {
   int next_index() const;
   int previous_index() const;
   bool stop_after_current() const;
+  bool is_dynamic() const { return dynamic_playlist_; }
 
-  const boost::shared_ptr<PlaylistItem>& item_at(int index) const { return items_[index]; }
-  boost::shared_ptr<PlaylistItem> current_item() const { return current_item_; }
+  const PlaylistItemPtr& item_at(int index) const { return items_[index]; }
+  PlaylistItemPtr current_item() const { return current_item_; }
 
   PlaylistItem::Options current_item_options() const;
   Song current_item_metadata() const;
@@ -140,9 +152,10 @@ class Playlist : public QAbstractListModel {
   // Changing the playlist
   QModelIndex InsertItems(const PlaylistItemList& items, int pos = -1);
   QModelIndex InsertLibraryItems(const SongList& items, int pos = -1);
-  QModelIndex InsertMagnatuneItems(const SongList& items, int pos = -1);
   QModelIndex InsertSongs(const SongList& items, int pos = -1);
+  QModelIndex InsertSongsOrLibraryItems(const SongList& items, int pos = -1);
   QModelIndex InsertRadioStations(const QList<RadioItem*>& items, int pos = -1, bool play_now = false);
+  void InsertSmartPlaylist(smart_playlists::GeneratorPtr generator, int pos = -1, bool play_now = false);
   void InsertUrls(const QList<QUrl>& urls, bool play_now, int pos = -1);
   void StopAfter(int row);
   void ReloadItems(const QList<int>& rows);
@@ -161,7 +174,6 @@ class Playlist : public QAbstractListModel {
   void sort(int column, Qt::SortOrder order);
   bool removeRows(int row, int count, const QModelIndex& parent = QModelIndex());
 
-
  public slots:
   void set_current_index(int index);
   void Paused();
@@ -171,18 +183,25 @@ class Playlist : public QAbstractListModel {
 
   void ClearStreamMetadata();
   void SetStreamMetadata(const QUrl& url, const Song& song);
+  void ItemChanged(PlaylistItemPtr item);
 
   void Clear();
   void Shuffle();
 
   void ShuffleModeChanged(PlaylistSequence::ShuffleMode mode);
 
+  void RepopulateDynamicPlaylist();
+  void TurnOffDynamicPlaylist();
+
  signals:
   void CurrentSongChanged(const Song& metadata);
   void EditingFinished(const QModelIndex& index);
   void PlayRequested(const QModelIndex& index);
 
+  // Signals that the underlying list of items was changed, meaning that
+  // something was added to it, removed from it or the ordering changed.
   void PlaylistChanged();
+  void DynamicModeChanged(bool dynamic);
 
   void LoadTracksError(const QString& message);
 
@@ -193,6 +212,7 @@ class Playlist : public QAbstractListModel {
   int NextVirtualIndex(int i) const;
   int PreviousVirtualIndex(int i) const;
   bool FilterContainsVirtualIndex(int i) const;
+  void TurnOnDynamicPlaylist(smart_playlists::GeneratorPtr gen);
 
   // Modify the playlist without changing the undo stack.  These are used by
   // our friends in PlaylistUndoCommands
@@ -201,6 +221,8 @@ class Playlist : public QAbstractListModel {
   void MoveItemsWithoutUndo(const QList<int>& source_rows, int pos);
   void MoveItemsWithoutUndo(int start, const QList<int>& dest_rows);
 
+  void RemoveItemsNotInQueue();
+
  private slots:
   void TracksAboutToBeDequeued(const QModelIndex&, int begin, int end);
   void TracksDequeued();
@@ -211,6 +233,7 @@ class Playlist : public QAbstractListModel {
   void ItemsLoaded();
 
  private:
+  bool is_loading_;
   PlaylistFilter* proxy_;
   Queue* queue_;
 
@@ -226,7 +249,7 @@ class Playlist : public QAbstractListModel {
                              // that they will be played.
   // A map of library ID to playlist item - for fast lookups when library
   // items change.
-  QMultiMap<int, boost::shared_ptr<PlaylistItem> > library_items_by_id_;
+  QMultiMap<int, PlaylistItemPtr> library_items_by_id_;
 
   QPersistentModelIndex current_item_index_;
   QPersistentModelIndex last_played_item_index_;
@@ -234,7 +257,7 @@ class Playlist : public QAbstractListModel {
   bool current_is_paused_;
   int current_virtual_index_;
 
-  boost::shared_ptr<PlaylistItem> current_item_;
+  PlaylistItemPtr current_item_;
 
   bool is_shuffled_;
 
@@ -247,6 +270,8 @@ class Playlist : public QAbstractListModel {
   bool ignore_sorting_;
 
   QUndoStack* undo_stack_;
+
+  smart_playlists::GeneratorPtr dynamic_playlist_;
 };
 
 QDataStream& operator <<(QDataStream&, const Playlist*);
