@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -158,10 +159,12 @@ void PlaylistContainer::SetViewModel(Playlist* playlist) {
   view()->setModel(playlist->proxy());
   view()->SetItemDelegates(manager_->library_backend());
   view()->SetPlaylist(playlist);
+  view()->selectionModel()->select(manager_->current_selection(), QItemSelectionModel::ClearAndSelect);
   playlist->IgnoreSorting(false);
 
   connect(view()->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
           this, SLOT(SelectionChanged()));
+  emit ViewSelectionModelChanged();
 
   // Update filter
   filter_->setText(playlist->proxy()->filterRegExp().pattern());
