@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,9 +31,11 @@ class PlaylistHeader : public StretchHeaderView {
 
   // QWidget
   void contextMenuEvent(QContextMenuEvent* e);
+  void enterEvent(QEvent*);
 
  signals:
   void SectionVisibilityChanged(int logical, bool visible);
+  void MouseEntered();
 
  private slots:
   void HideCurrent();
