@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,8 +17,12 @@
 
 #include "playlistitem.h"
 #include "songplaylistitem.h"
+#include "library/library.h"
 #include "library/libraryplaylistitem.h"
+#include "radio/jamendoplaylistitem.h"
+#include "radio/jamendoservice.h"
 #include "radio/magnatuneplaylistitem.h"
+#include "radio/magnatuneservice.h"
 #include "radio/radioplaylistitem.h"
 
 #include <QtConcurrentRun>
@@ -34,6 +39,8 @@ PlaylistItem* PlaylistItem::NewFromType(const QString& type) {
     return new LibraryPlaylistItem(type);
   if (type == "Magnatune")
     return new MagnatunePlaylistItem(type);
+  if (type == "Jamendo")
+    return new JamendoPlaylistItem(type);
   if (type == "Stream" || type == "File")
     return new SongPlaylistItem(type);
   if (type == "Radio")
@@ -43,6 +50,18 @@ PlaylistItem* PlaylistItem::NewFromType(const QString& type) {
   return NULL;
 }
 
+PlaylistItem* PlaylistItem::NewFromSongsTable(const QString& table, const Song& song) {
+  if (table == Library::kSongsTable)
+    return new LibraryPlaylistItem(song);
+  if (table == MagnatuneService::kSongsTable)
+    return new MagnatunePlaylistItem(song);
+  if (table == JamendoService::kSongsTable)
+    return new JamendoPlaylistItem(song);
+
+  qWarning() << "Invalid PlaylistItem songs table:" << table;
+  return NULL;
+}
+
 void PlaylistItem::BindToQuery(QSqlQuery* query) const {
   query->bindValue(1, type());
   query->bindValue(2, DatabaseValue(Column_LibraryId));
@@ -63,12 +82,10 @@ void PlaylistItem::ClearTemporaryMetadata() {
   temp_metadata_ = Song();
 }
 
-static void ReloadPlaylistItem(boost::shared_ptr<PlaylistItem> item) {
+static void ReloadPlaylistItem(PlaylistItemPtr item) {
   item->Reload();
 }
 
 QFuture<void> PlaylistItem::BackgroundReload() {
   return QtConcurrent::run(ReloadPlaylistItem, shared_from_this());
 }
-
-
