@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,17 +23,20 @@
 #include "library/librarybackend.h"
 #include "library/libraryplaylistitem.h"
 #include "playlistparsers/playlistparser.h"
+#include "smartplaylists/generator.h"
 
 #include <QFileInfo>
 #include <QtDebug>
 
+using smart_playlists::GeneratorPtr;
+
 PlaylistManager::PlaylistManager(TaskManager* task_manager, QObject *parent)
   : QObject(parent),
     task_manager_(task_manager),
     playlist_backend_(NULL),
     library_backend_(NULL),
     sequence_(NULL),
-    parser_(new PlaylistParser(this)),
+    parser_(NULL),
     current_(-1),
     active_(-1)
 {
@@ -50,8 +54,10 @@ void PlaylistManager::Init(LibraryBackend* library_backend,
   library_backend_ = library_backend;
   playlist_backend_ = playlist_backend;
   sequence_ = sequence;
+  parser_ = new PlaylistParser(library_backend, this);
 
   connect(library_backend_, SIGNAL(SongsDiscovered(SongList)), SLOT(SongsDiscovered(SongList)));
+  connect(library_backend_, SIGNAL(SongsStatisticsChanged(SongList)), SLOT(SongsDiscovered(SongList)));
 
   foreach (const PlaylistBackend::Playlist& p, playlist_backend->GetAllPlaylists()) {
     AddPlaylist(p.id, p.name);
@@ -60,6 +66,8 @@ void PlaylistManager::Init(LibraryBackend* library_backend,
   // If no playlist exists then make a new one
   if (playlists_.isEmpty())
     New(tr("Playlist"));
+
+  emit PlaylistManagerInitialized();
 }
 
 Playlist* PlaylistManager::AddPlaylist(int id, const QString& name) {
@@ -67,7 +75,7 @@ Playlist* PlaylistManager::AddPlaylist(int id, const QString& name) {
   ret->set_sequence(sequence_);
 
   connect(ret, SIGNAL(CurrentSongChanged(Song)), SIGNAL(CurrentSongChanged(Song)));
-  connect(ret, SIGNAL(PlaylistChanged()), SIGNAL(PlaylistChanged()));
+  connect(ret, SIGNAL(PlaylistChanged()), SLOT(OneOfPlaylistsChanged()));
   connect(ret, SIGNAL(PlaylistChanged()), SLOT(UpdateSummaryText()));
   connect(ret, SIGNAL(EditingFinished(QModelIndex)), SIGNAL(EditingFinished(QModelIndex)));
   connect(ret, SIGNAL(LoadTracksError(QString)), SIGNAL(Error(QString)));
@@ -94,7 +102,7 @@ void PlaylistManager::New(const QString& name, const SongList& songs) {
     qFatal("Couldn't create playlist");
 
   Playlist* playlist = AddPlaylist(id, name);
-  playlist->InsertSongs(songs);
+  playlist->InsertSongsOrLibraryItems(songs);
 
   SetCurrentPlaylist(id);
 }
@@ -170,6 +178,10 @@ void PlaylistManager::Remove(int id) {
   emit PlaylistRemoved(id);
 }
 
+void PlaylistManager::OneOfPlaylistsChanged() {
+  emit PlaylistChanged(qobject_cast<Playlist*>(sender()));
+}
+
 void PlaylistManager::SetCurrentPlaylist(int id) {
   Q_ASSERT(playlists_.contains(id));
   current_ = id;
@@ -187,6 +199,8 @@ void PlaylistManager::SetActivePlaylist(int id) {
 
   active_ = id;
   emit ActiveChanged(active());
+
+  sequence_->SetUsingDynamicPlaylist(active()->is_dynamic());
 }
 
 void PlaylistManager::ClearCurrent() {
@@ -223,7 +237,7 @@ void PlaylistManager::UpdateSummaryText() {
   int selected = 0;
 
   // Get the length of the selected tracks
-  foreach (const QItemSelectionRange& range, current_selection_) {
+  foreach (const QItemSelectionRange& range, playlists_[current_id()].selection) {
     if (!range.isValid())
       continue;
 
@@ -251,8 +265,8 @@ void PlaylistManager::UpdateSummaryText() {
   emit SummaryTextChanged(summary);
 }
 
-void PlaylistManager::SelectionChanged(const QItemSelection &selection) {
-  current_selection_ = selection;
+void PlaylistManager::SelectionChanged(const QItemSelection& selection) {
+  playlists_[current_id()].selection = selection;
   UpdateSummaryText();
 }
 
@@ -263,11 +277,24 @@ void PlaylistManager::SongsDiscovered(const SongList& songs) {
   foreach (const Song& song, songs) {
     foreach (const Data& data, playlists_) {
       PlaylistItemList items = data.p->library_items_by_id(song.id());
-      foreach (boost::shared_ptr<PlaylistItem> item, items) {
+      foreach (PlaylistItemPtr item, items) {
         if (item->Metadata().directory_id() != song.directory_id())
           continue;
         static_cast<LibraryPlaylistItem*>(item.get())->SetMetadata(song);
+        data.p->ItemChanged(item);
       }
     }
   }
 }
+
+void PlaylistManager::PlaySmartPlaylist(GeneratorPtr generator, bool as_new, bool clear) {
+  if (as_new) {
+    New(generator->name());
+  }
+
+  if (clear) {
+    current()->Clear();
+  }
+
+  current()->InsertSmartPlaylist(generator);
+}
