@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,6 +15,7 @@
    along with Clementine.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include "dynamicplaylistcontrols.h"
 #include "playlistview.h"
 #include "playlist.h"
 #include "playlistheader.h"
@@ -29,10 +31,12 @@
 #include <QKeyEvent>
 #include <QApplication>
 #include <QSortFilterProxyModel>
+#include <QScrollBar>
 
 #include <math.h>
 
 const char* PlaylistView::kSettingsGroup = "Playlist";
+const int PlaylistView::kStateVersion = 3;
 const int PlaylistView::kGlowIntensitySteps = 24;
 const int PlaylistView::kAutoscrollGraceTimeout = 60; // seconds
 const int PlaylistView::kDropIndicatorWidth = 2;
@@ -54,7 +58,8 @@ void PlaylistProxyStyle::drawControl(
 }
 
 void PlaylistProxyStyle::drawPrimitive(PrimitiveElement element, const QStyleOption* option, QPainter* painter, const QWidget* widget) const {
-  if (element == QStyle::PE_PanelItemViewRow)
+  if (element == QStyle::PE_PanelItemViewRow ||
+      element == QStyle::PE_PanelItemViewItem)
     cleanlooks_->drawPrimitive(element, option, painter, widget);
   else
     QProxyStyle::drawPrimitive(element, option, painter, widget);
@@ -67,9 +72,11 @@ PlaylistView::PlaylistView(QWidget *parent)
     playlist_(NULL),
     header_(new PlaylistHeader(Qt::Horizontal, this)),
     setting_initial_header_layout_(false),
+    read_only_settings_(false),
     glow_enabled_(true),
     currently_glowing_(false),
     glow_intensity_step_(0),
+    rating_delegate_(NULL),
     inhibit_autoscroll_timer_(new QTimer(this)),
     inhibit_autoscroll_(false),
     currently_autoscrolling_(false),
@@ -77,11 +84,14 @@ PlaylistView::PlaylistView(QWidget *parent)
     currenttrack_play_(":currenttrack_play.png"),
     currenttrack_pause_(":currenttrack_pause.png"),
     cached_current_row_row_(-1),
-    drop_indicator_row_(-1)
+    drop_indicator_row_(-1),
+    drag_over_(false),
+    dynamic_controls_(new DynamicPlaylistControls(this))
 {
   setHeader(header_);
   header_->setMovable(true);
   setStyle(style_);
+  setMouseTracking(true);
 
   connect(header_, SIGNAL(sectionResized(int,int,int)), SLOT(SaveGeometry()));
   connect(header_, SIGNAL(sectionMoved(int,int,int)), SLOT(SaveGeometry()));
@@ -91,21 +101,29 @@ PlaylistView::PlaylistView(QWidget *parent)
   connect(header_, SIGNAL(SectionVisibilityChanged(int,bool)), SLOT(InvalidateCachedCurrentPixmap()));
   connect(header_, SIGNAL(StretchEnabledChanged(bool)), SLOT(SaveSettings()));
   connect(header_, SIGNAL(StretchEnabledChanged(bool)), SLOT(StretchChanged(bool)));
+  connect(header_, SIGNAL(MouseEntered()), SLOT(RatingHoverOut()));
 
   inhibit_autoscroll_timer_->setInterval(kAutoscrollGraceTimeout * 1000);
   inhibit_autoscroll_timer_->setSingleShot(true);
   connect(inhibit_autoscroll_timer_, SIGNAL(timeout()), SLOT(InhibitAutoscrollTimeout()));
 
+  horizontalScrollBar()->installEventFilter(this);
+  verticalScrollBar()->installEventFilter(this);
+
   setAlternatingRowColors(true);
 
   setAttribute(Qt::WA_MacShowFocusRect, false);
 
+  dynamic_controls_->hide();
+
 #ifdef Q_OS_DARWIN
   setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
 #endif
 }
 
 void PlaylistView::SetItemDelegates(LibraryBackend* backend) {
+  rating_delegate_ = new RatingItemDelegate(this);
+
   setItemDelegate(new PlaylistDelegateBase(this));
   setItemDelegateForColumn(Playlist::Column_Title, new TextItemDelegate(this));
   setItemDelegateForColumn(Playlist::Column_Album,
@@ -123,21 +141,34 @@ void PlaylistView::SetItemDelegates(LibraryBackend* backend) {
   setItemDelegateForColumn(Playlist::Column_Samplerate, new PlaylistDelegateBase(this, ("Hz")));
   setItemDelegateForColumn(Playlist::Column_Bitrate, new PlaylistDelegateBase(this, tr("kbps")));
   setItemDelegateForColumn(Playlist::Column_Filename, new NativeSeparatorsDelegate(this));
+  setItemDelegateForColumn(Playlist::Column_Rating, rating_delegate_);
+  setItemDelegateForColumn(Playlist::Column_LastPlayed, new LastPlayedItemDelegate(this));
 }
 
 void PlaylistView::SetPlaylist(Playlist *playlist) {
   if (playlist_) {
     disconnect(playlist_, SIGNAL(CurrentSongChanged(Song)),
                this, SLOT(MaybeAutoscroll()));
+    disconnect(playlist_, SIGNAL(DynamicModeChanged(bool)),
+               this, SLOT(DynamicModeChanged(bool)));
     disconnect(playlist_, SIGNAL(destroyed()), this, SLOT(PlaylistDestroyed()));
+
+    disconnect(dynamic_controls_, SIGNAL(Repopulate()),
+               playlist_, SLOT(RepopulateDynamicPlaylist()));
+    disconnect(dynamic_controls_, SIGNAL(TurnOff()),
+               playlist_, SLOT(TurnOffDynamicPlaylist()));
   }
 
   playlist_ = playlist;
   LoadGeometry();
   ReloadSettings();
+  DynamicModeChanged(playlist->is_dynamic());
 
   connect(playlist_, SIGNAL(CurrentSongChanged(Song)), SLOT(MaybeAutoscroll()));
-  connect(playlist_, SIGNAL(destroyed()), this, SLOT(PlaylistDestroyed()));
+  connect(playlist_, SIGNAL(DynamicModeChanged(bool)), SLOT(DynamicModeChanged(bool)));
+  connect(playlist_, SIGNAL(destroyed()), SLOT(PlaylistDestroyed()));
+  connect(dynamic_controls_, SIGNAL(Repopulate()), playlist_, SLOT(RepopulateDynamicPlaylist()));
+  connect(dynamic_controls_, SIGNAL(TurnOff()), playlist_, SLOT(TurnOffDynamicPlaylist()));
 }
 
 void PlaylistView::setModel(QAbstractItemModel *m) {
@@ -170,16 +201,52 @@ void PlaylistView::LoadGeometry() {
     header_->HideSection(Playlist::Column_DateModified);
     header_->HideSection(Playlist::Column_AlbumArtist);
     header_->HideSection(Playlist::Column_Composer);
+    header_->HideSection(Playlist::Column_Rating);
+    header_->HideSection(Playlist::Column_PlayCount);
+    header_->HideSection(Playlist::Column_SkipCount);
+    header_->HideSection(Playlist::Column_LastPlayed);
 
     header_->moveSection(header_->visualIndex(Playlist::Column_Track), 0);
     setting_initial_header_layout_ = true;
   }
+
+  // New columns that we add are visible by default if the user has upgraded
+  // Clementine.  Hide them again here
+  const int state_version = settings.value("state_version", 0).toInt();
+  if (state_version < 1) {
+    header_->HideSection(Playlist::Column_Rating);
+    header_->HideSection(Playlist::Column_PlayCount);
+    header_->HideSection(Playlist::Column_SkipCount);
+    header_->HideSection(Playlist::Column_LastPlayed);
+  }
+  if (state_version < 2) {
+    header_->HideSection(Playlist::Column_Score);
+  }
+  if (state_version < 3) {
+    header_->HideSection(Playlist::Column_Comment);
+  }
+
+  // Make sure at least one column is visible
+  bool all_hidden = true;
+  for (int i=0 ; i<header_->count() ; ++i) {
+    if (!header_->isSectionHidden(i) && header_->sectionSize(i) > 0) {
+      all_hidden = false;
+      break;
+    }
+  }
+  if (all_hidden) {
+    header_->ShowSection(Playlist::Column_Title);
+  }
 }
 
 void PlaylistView::SaveGeometry() {
+  if (read_only_settings_)
+    return;
+
   QSettings settings;
   settings.beginGroup(kSettingsGroup);
   settings.setValue("state", header_->saveState());
+  settings.setValue("state_version", kStateVersion);
 }
 
 void PlaylistView::ReloadBarPixmaps() {
@@ -214,6 +281,12 @@ QList<QPixmap> PlaylistView::LoadBarPixmap(const QString& filename) {
   return ret;
 }
 
+void PlaylistView::drawTree(QPainter* painter, const QRegion& region) const {
+  const_cast<PlaylistView*>(this)->current_paint_region_ = region;
+  QTreeView::drawTree(painter, region);
+  const_cast<PlaylistView*>(this)->current_paint_region_ = QRegion();
+}
+
 void PlaylistView::drawRow(QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index) const {
   QStyleOptionViewItemV4 opt(option);
 
@@ -264,13 +337,26 @@ void PlaylistView::drawRow(QPainter* painter, const QStyleOptionViewItem& option
 
     // Draw the actual row data on top.  We cache this, because it's fairly
     // expensive (1-2ms), and we do it many times per second.
-    if (cached_current_row_rect_ != opt.rect ||
-        cached_current_row_row_ != index.row() ||
-        cached_current_row_.isNull()) {
-      const_cast<PlaylistView*>(this)->UpdateCachedCurrentRowPixmap(opt, index);
-    }
+    const bool cache_dirty = cached_current_row_rect_ != opt.rect ||
+                             cached_current_row_row_ != index.row() ||
+                             cached_current_row_.isNull();
 
-    painter->drawPixmap(opt.rect, cached_current_row_);
+    // We can't update the cache if we're not drawing the entire region,
+    // QTreeView clips its drawing to only the columns in the region, so it
+    // wouldn't update the whole pixmap properly.
+    const bool whole_region =
+        current_paint_region_.boundingRect().width() == viewport()->width();
+
+    if (!cache_dirty) {
+      painter->drawPixmap(opt.rect, cached_current_row_);
+    } else {
+      if (whole_region) {
+        const_cast<PlaylistView*>(this)->UpdateCachedCurrentRowPixmap(opt, index);
+        painter->drawPixmap(opt.rect, cached_current_row_);
+      } else {
+        QTreeView::drawRow(painter, opt, index);
+      }
+    }
   } else {
     QTreeView::drawRow(painter, opt, index);
   }
@@ -287,7 +373,6 @@ void PlaylistView::UpdateCachedCurrentRowPixmap(QStyleOptionViewItemV4 option,
 
   QPainter p(&cached_current_row_);
   QTreeView::drawRow(&p, option, index);
-  p.end();
 }
 
 void PlaylistView::InvalidateCachedCurrentPixmap() {
@@ -436,8 +521,97 @@ void PlaylistView::closeEditor(QWidget* editor, QAbstractItemDelegate::EndEditHi
   }
 }
 
-void PlaylistView::mousePressEvent(QMouseEvent *event) {
-  QTreeView::mousePressEvent(event);
+void PlaylistView::mouseMoveEvent(QMouseEvent* event) {
+  QModelIndex index = indexAt(event->pos());
+  if (index.isValid() && index.data(Playlist::Role_CanSetRating).toBool()) {
+    RatingHoverIn(index, event->pos());
+  } else if (rating_delegate_->is_mouse_over()) {
+    RatingHoverOut();
+  }
+  if (!drag_over_) {
+    QTreeView::mouseMoveEvent(event);
+  }
+}
+
+void PlaylistView::leaveEvent(QEvent* e) {
+  if (rating_delegate_->is_mouse_over()) {
+    RatingHoverOut();
+  }
+  QTreeView::leaveEvent(e);
+}
+
+void PlaylistView::RatingHoverIn(const QModelIndex& index, const QPoint& pos) {
+  if (!(editTriggers() & QAbstractItemView::SelectedClicked)) {
+    return;
+  }
+
+  const QModelIndex old_index = rating_delegate_->mouse_over_index();
+  rating_delegate_->set_mouse_over(index, selectedIndexes(), pos);
+  setCursor(Qt::PointingHandCursor);
+
+  update(index);
+  update(old_index);
+  foreach (const QModelIndex& index, selectedIndexes()) {
+    if (index.column() == Playlist::Column_Rating) {
+      update(index);
+    }
+  }
+
+  if (index.data(Playlist::Role_IsCurrent).toBool() ||
+      old_index.data(Playlist::Role_IsCurrent).toBool()) {
+    InvalidateCachedCurrentPixmap();
+  }
+}
+
+void PlaylistView::RatingHoverOut() {
+  if (!(editTriggers() & QAbstractItemView::SelectedClicked)) {
+    return;
+  }
+
+  const QModelIndex old_index = rating_delegate_->mouse_over_index();
+  rating_delegate_->set_mouse_out();
+  setCursor(QCursor());
+
+  update(old_index);
+  foreach (const QModelIndex& index, selectedIndexes()) {
+    if (index.column() == Playlist::Column_Rating) {
+      update(index);
+    }
+  }
+
+  if (old_index.data(Playlist::Role_IsCurrent).toBool()) {
+    InvalidateCachedCurrentPixmap();
+  }
+}
+
+void PlaylistView::mousePressEvent(QMouseEvent* event) {
+  if (!(editTriggers() & QAbstractItemView::SelectedClicked)) {
+    QTreeView::mousePressEvent(event);
+    return;
+  }
+
+  QModelIndex index = indexAt(event->pos());
+  if (event->button() == Qt::LeftButton && index.isValid() &&
+      index.data(Playlist::Role_CanSetRating).toBool()) {
+    // Calculate which star was clicked
+    double new_rating = RatingPainter::RatingForPos(
+        event->pos(), visualRect(index));
+
+    if (selectedIndexes().contains(index)) {
+      // Update all the selected items
+      foreach (const QModelIndex& index, selectedIndexes()) {
+        if (index.data(Playlist::Role_CanSetRating).toBool()) {
+          emit SongRatingSet(index, new_rating);
+        }
+      }
+    } else {
+      // Update only this item
+      emit SongRatingSet(index, new_rating);
+    }
+  } else {
+    QTreeView::mousePressEvent(event);
+  }
+
   inhibit_autoscroll_ = true;
   inhibit_autoscroll_timer_->start();
 }
@@ -569,11 +743,13 @@ void PlaylistView::dragMoveEvent(QDragMoveEvent *event) {
 void PlaylistView::dragEnterEvent(QDragEnterEvent *event) {
   QTreeView::dragEnterEvent(event);
   cached_tree_ = QPixmap();
+  drag_over_ = true;
 }
 
 void PlaylistView::dragLeaveEvent(QDragLeaveEvent *event) {
   QTreeView::dragLeaveEvent(event);
   cached_tree_ = QPixmap();
+  drag_over_ = false;
   drop_indicator_row_ = -1;
 }
 
@@ -581,6 +757,7 @@ void PlaylistView::dropEvent(QDropEvent *event) {
   QTreeView::dropEvent(event);
   cached_tree_ = QPixmap();
   drop_indicator_row_ = -1;
+  drag_over_ = false;
 }
 
 void PlaylistView::PlaylistDestroyed() {
@@ -607,6 +784,9 @@ void PlaylistView::ReloadSettings() {
 }
 
 void PlaylistView::SaveSettings() {
+  if (read_only_settings_)
+    return;
+
   QSettings s;
   s.beginGroup(kSettingsGroup);
   s.setValue("glow_effect", glow_enabled_);
@@ -616,3 +796,34 @@ void PlaylistView::SaveSettings() {
 void PlaylistView::StretchChanged(bool stretch) {
   setHorizontalScrollBarPolicy(stretch ? Qt::ScrollBarAlwaysOff : Qt::ScrollBarAsNeeded);
 }
+
+void PlaylistView::DynamicModeChanged(bool dynamic) {
+  if (!dynamic) {
+    dynamic_controls_->hide();
+  } else {
+    RepositionDynamicControls();
+    dynamic_controls_->show();
+  }
+}
+
+void PlaylistView::resizeEvent(QResizeEvent* e) {
+  QTreeView::resizeEvent(e);
+  if (dynamic_controls_->isVisible()) {
+    RepositionDynamicControls();
+  }
+}
+
+void PlaylistView::RepositionDynamicControls() {
+  dynamic_controls_->resize(dynamic_controls_->sizeHint());
+  dynamic_controls_->move((width() - dynamic_controls_->width()) / 2,
+                          height() - dynamic_controls_->height() - 20);
+}
+
+bool PlaylistView::eventFilter(QObject* object, QEvent* event) {
+  if (event->type() == QEvent::Enter &&
+      (object == horizontalScrollBar() || object == verticalScrollBar())) {
+    RatingHoverOut();
+    return false;
+  }
+  return QObject::eventFilter(object, event);
+}
