@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +24,7 @@ class ASXParser : public XMLParser {
   Q_OBJECT
 
  public:
-  ASXParser(QObject* parent = 0);
+  ASXParser(LibraryBackendInterface* library, QObject* parent = 0);
 
   QString name() const { return "ASX"; }
   QStringList file_extensions() const { return QStringList() << "asx"; }
