@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,7 +29,7 @@ class XSPFParser : public XMLParser {
   Q_OBJECT
 
  public:
-  XSPFParser(QObject* parent = 0);
+  XSPFParser(LibraryBackendInterface* library, QObject* parent = 0);
 
   QString name() const { return "XSPF"; }
   QStringList file_extensions() const { return QStringList() << "xspf"; }
