@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
 #include "radioitem.h"
 #include "lastfmstationdialog.h"
 #include "radiomodel.h"
-#include "core/networkaccessmanager.h"
+#include "radioplaylistitem.h"
 #include "core/song.h"
 #include "core/taskmanager.h"
 #include "ui/iconloader.h"
@@ -60,14 +61,16 @@ LastFMService::LastFMService(RadioModel* parent)
     scrobbling_enabled_(false),
     artist_list_(NULL),
     tag_list_(NULL),
+    custom_list_(NULL),
     friends_list_(NULL),
-    neighbours_list_(NULL),
-    network_(parent->network()->network())
+    neighbours_list_(NULL)
 {
   ReloadSettings();
 
   play_action_ = context_menu_->addAction(
       IconLoader::Load("media-playback-start"), tr("Add to playlist"), this, SLOT(AddToPlaylist()));
+  load_action_ = context_menu_->addAction(
+      IconLoader::Load("media-playback-start"), tr("Load"), this, SLOT(LoadToPlaylist()));
   remove_action_ = context_menu_->addAction(
       IconLoader::Load("list-remove"), tr("Remove"), this, SLOT(Remove()));
   context_menu_->addSeparator();
@@ -75,12 +78,15 @@ LastFMService::LastFMService(RadioModel* parent)
       QIcon(":last.fm/icon_radio.png"), tr("Play artist radio..."), this, SLOT(AddArtistRadio()));
   add_tag_action_ = context_menu_->addAction(
       QIcon(":last.fm/icon_tag.png"), tr("Play tag radio..."), this, SLOT(AddTagRadio()));
+  add_custom_action_ = context_menu_->addAction(
+      QIcon(":last.fm/icon_radio.png"), tr("Play custom radio..."), this, SLOT(AddCustomRadio()));
   context_menu_->addAction(
       IconLoader::Load("configure"), tr("Configure Last.fm..."), this, SLOT(ShowConfig()));
 
   remove_action_->setEnabled(false);
   add_artist_action_->setEnabled(false);
   add_tag_action_->setEnabled(false);
+  add_custom_action_->setEnabled(false);
 }
 
 LastFMService::~LastFMService() {
@@ -118,7 +124,7 @@ void LastFMService::LazyPopulate(RadioItem *item) {
       // Normal radio types
       CreateStationItem(Type_MyRecommendations, tr("My Recommendations"), ":last.fm/recommended_radio.png", item);
       CreateStationItem(Type_MyRadio, tr("My Radio Station"), ":last.fm/personal_radio.png", item);
-      CreateStationItem(Type_MyLoved, tr("My Loved Tracks"), ":last.fm/loved_radio.png", item);
+      CreateStationItem(Type_MyMix, tr("My Mix Radio"), ":last.fm/loved_radio.png", item);
       CreateStationItem(Type_MyNeighbourhood, tr("My Neighborhood"), ":last.fm/neighbour_radio.png", item);
 
       // Types that have children
@@ -130,8 +136,13 @@ void LastFMService::LazyPopulate(RadioItem *item) {
       tag_list_->icon = QIcon(":last.fm/icon_tag.png");
       tag_list_->lazy_loaded = true;
 
+      custom_list_ = new RadioItem(this, Type_CustomRadio, tr("Custom radio"), item);
+      custom_list_->icon = QIcon(":last.fm/icon_radio.png");
+      custom_list_->lazy_loaded = true;
+
       RestoreList("artists", Type_Artist, QIcon(":last.fm/icon_radio.png"), artist_list_);
       RestoreList("tags", Type_Tag, QIcon(":last.fm/icon_tag.png"), tag_list_);
+      RestoreList("custom", Type_Custom, QIcon(":last.fm/icon_radio.png"), custom_list_);
 
       friends_list_ = new RadioItem(this, Type_MyFriends, tr("Friends"), item);
       friends_list_->icon = QIcon(":last.fm/my_friends.png");
@@ -144,6 +155,7 @@ void LastFMService::LazyPopulate(RadioItem *item) {
 
       add_artist_action_->setEnabled(true);
       add_tag_action_->setEnabled(true);
+      add_custom_action_->setEnabled(true);
       break;
 
     case Type_MyFriends:
@@ -157,8 +169,8 @@ void LastFMService::LazyPopulate(RadioItem *item) {
     case Type_OtherUser:
       CreateStationItem(Type_OtherUserRadio, item->key, ":last.fm/recommended_radio.png", item)
           ->display_text = tr("Last.fm Radio Station - %1").arg(item->key);
-      CreateStationItem(Type_OtherUserLoved, item->key, ":last.fm/loved_radio.png", item)
-          ->display_text = tr("Last.fm Loved Tracks - %1").arg(item->key);
+      CreateStationItem(Type_OtherUserMix, item->key, ":last.fm/loved_radio.png", item)
+          ->display_text = tr("Last.fm Mix Radio - %1").arg(item->key);
       CreateStationItem(Type_OtherUserNeighbourhood, item->key, ":last.fm/neighbour_radio.png", item)
           ->display_text = tr("Last.fm Neighbor Radio - %1").arg(item->key);
       break;
@@ -241,8 +253,8 @@ QUrl LastFMService::UrlForItem(const RadioItem* item) const {
     case Type_MyRecommendations:
       return "lastfm://user/" + lastfm::ws::Username + "/recommended";
 
-    case Type_MyLoved:
-      return "lastfm://user/" + lastfm::ws::Username + "/loved";
+    case Type_MyMix:
+      return QString("lastfm://rql/") + QString("adv:" + lastfm::ws::Username).toUtf8().toBase64();
 
     case Type_MyNeighbourhood:
       return "lastfm://user/" + lastfm::ws::Username + "/neighbours";
@@ -254,8 +266,8 @@ QUrl LastFMService::UrlForItem(const RadioItem* item) const {
     case Type_OtherUserRadio:
       return "lastfm://user/" + item->key + "/library";
 
-    case Type_OtherUserLoved:
-      return "lastfm://user/" + item->key + "/loved";
+    case Type_OtherUserMix:
+      return QString("lastfm://rql/") + QString("adv:" + item->key).toUtf8().toBase64();
 
     case Type_OtherUserNeighbourhood:
       return "lastfm://user/" + item->key + "/neighbours";
@@ -265,6 +277,9 @@ QUrl LastFMService::UrlForItem(const RadioItem* item) const {
 
     case Type_Tag:
       return "lastfm://globaltags/" + item->key;
+
+    case Type_Custom:
+      return QString("lastfm://rql/" + item->key.toUtf8().toBase64());
   }
   return QUrl();
 }
@@ -274,15 +289,16 @@ QString LastFMService::TitleForItem(const RadioItem* item) const {
 
   switch (item->type) {
     case Type_MyRecommendations: return tr("Last.fm Recommended Radio - %1").arg(me);
-    case Type_MyLoved:           return tr("Last.fm Loved Tracks - %1").arg(me);
+    case Type_MyMix:             return tr("Last.fm Mix Radio - %1").arg(me);
     case Type_MyNeighbourhood:   return tr("Last.fm Neighbor Radio - %1").arg(me);
     case Type_MyRadio:           return tr("Last.fm Library - %1").arg(me);
     case Type_OtherUser:
     case Type_OtherUserRadio:    return tr("Last.fm Library - %1").arg(item->key);
-    case Type_OtherUserLoved:    return tr("Last.fm Loved Tracks - %1").arg(item->key);
+    case Type_OtherUserMix:      return tr("Last.fm Mix Radio - %1").arg(item->key);
     case Type_OtherUserNeighbourhood: return tr("Last.fm Neighbor Radio - %1").arg(item->key);
     case Type_Artist:            return tr("Last.fm Similar Artists to %1").arg(item->key);
     case Type_Tag:               return tr("Last.fm Tag Radio: %1").arg(item->key);
+    case Type_Custom:            return tr("Last.fm Custom Radio: %1").arg(item->key);
   }
   return QString();
 }
@@ -455,6 +471,7 @@ void LastFMService::ShowContextMenu(RadioItem* item, const QModelIndex&,
   switch (item->type) {
     case Type_Artist:
     case Type_Tag:
+    case Type_Custom:
       remove_action_->setEnabled(true);
       break;
 
@@ -464,6 +481,7 @@ void LastFMService::ShowContextMenu(RadioItem* item, const QModelIndex&,
   }
 
   play_action_->setEnabled(item->playable);
+  load_action_->setEnabled(item->playable);
   context_menu_->popup(global_pos);
 }
 
@@ -542,7 +560,11 @@ void LastFMService::RefreshNeighboursFinished() {
 }
 
 void LastFMService::AddToPlaylist() {
-  emit AddItemToPlaylist(context_item_);
+  emit AddItemToPlaylist(context_item_, false);
+}
+
+void LastFMService::LoadToPlaylist() {
+  emit AddItemToPlaylist(context_item_, true);
 }
 
 void LastFMService::AddArtistRadio() {
@@ -553,6 +575,10 @@ void LastFMService::AddTagRadio() {
   AddArtistOrTag("tags", LastFMStationDialog::Tag, Type_Tag, QIcon(":last.fm/icon_tag.png"), tag_list_);
 }
 
+void LastFMService::AddCustomRadio() {
+  AddArtistOrTag("custom", LastFMStationDialog::Custom, Type_Custom, QIcon(":last.fm/icon_radio.png"), custom_list_);
+}
+
 void LastFMService::AddArtistOrTag(const QString& name,
                                    LastFMStationDialog::Type dialog_type, ItemType item_type,
                                    const QIcon& icon, RadioItem* list) {
@@ -568,7 +594,7 @@ void LastFMService::AddArtistOrTag(const QString& name,
   item->playable = true;
   item->lazy_loaded = true;
   item->InsertNotify(list);
-  emit AddItemToPlaylist(item);
+  emit AddItemToPlaylist(item, false);
 
   SaveList(name, list);
 }
@@ -613,6 +639,8 @@ void LastFMService::Remove() {
     SaveList("artists", artist_list_);
   else if (type == Type_Tag)
     SaveList("tags", tag_list_);
+  else if (type == Type_Custom)
+    SaveList("custom", custom_list_);
 }
 
 void LastFMService::FetchMoreTracks() {
@@ -705,3 +733,25 @@ PlaylistItem::Options LastFMService::playlistitem_options() const {
          PlaylistItem::PauseDisabled |
          PlaylistItem::ContainsMultipleTracks;
 }
+
+PlaylistItemPtr LastFMService::PlaylistItemForUrl(const QUrl& url) {
+  // This is a bit of a hack, it's only used by the artist/song info tag
+  // widgets for tag radio and similar artists radio.
+
+  PlaylistItemPtr ret;
+
+  if (url.scheme() != "lastfm")
+    return ret;
+
+  QStringList sections(url.path().split("/", QString::SkipEmptyParts));
+
+  if (sections.count() == 2 && url.host() == "artist" && sections[1] == "similarartists") {
+    ret.reset(new RadioPlaylistItem(this, url,
+        tr("Last.fm Similar Artists to %1").arg(sections[0]), QString()));
+  } else if (sections.count() == 1 && url.host() == "globaltags") {
+    ret.reset(new RadioPlaylistItem(this, url,
+        tr("Last.fm Tag Radio: %1").arg(sections[0]), QString()));
+  }
+
+  return ret;
+}
