@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -33,27 +34,33 @@
 class About;
 class AddStreamDialog;
 class AlbumCoverManager;
+class ArtistInfoView;
+class BackgroundStreams;
 class CommandlineOptions;
 class Database;
 class DeviceManager;
+class DeviceView;
 class EditTagDialog;
 class Equalizer;
 class ErrorDialog;
+class FileView;
 class GlobalShortcuts;
 class GroupByDialog;
 class Library;
+class LibraryViewContainer;
 class MultiLoadingIndicator;
-class NetworkAccessManager;
 class OrganiseDialog;
 class OSD;
 class Player;
 class PlaylistBackend;
 class PlaylistManager;
-class PlaylistParser;
 class QueueManager;
 class RadioItem;
 class RadioModel;
+class RadioViewContainer;
 class Song;
+class SongInfoBase;
+class SongInfoView;
 class SystemTrayIcon;
 class TaskManager;
 class TranscodeDialog;
@@ -67,7 +74,7 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   Q_OBJECT
 
  public:
-  MainWindow(NetworkAccessManager* network, Engine::Type engine, QWidget *parent = 0);
+  MainWindow(Engine::Type engine, QWidget *parent = 0);
   ~MainWindow();
 
   static const char* kSettingsGroup;
@@ -98,14 +105,18 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void MediaStopped();
   void MediaPaused();
   void MediaPlaying();
+  void TrackSkipped(PlaylistItemPtr item);
   void ForceShowOSD(const Song& song);
 
   void PlaylistRightClick(const QPoint& global_pos, const QModelIndex& index);
+  void PlaylistCurrentChanged(const QModelIndex& current);
+  void PlaylistViewSelectionModelChanged();
   void PlaylistPlay();
   void PlaylistStopAfter();
   void PlaylistQueue();
   void PlaylistRemoveCurrent();
   void PlaylistEditFinished(const QModelIndex& index);
+  void PlaylistSongRated(const QModelIndex& index, double rating);
   void EditTracks();
   void RenumberTracks();
   void SelectionSetValue();
@@ -121,6 +132,9 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void PlayIndex(const QModelIndex& index);
   void StopAfterCurrent();
 
+  void SongChanged(const Song& song);
+  void VolumeChanged(int volume);
+
   void LoadLibraryItemToPlaylist(const QModelIndexList& indexes);
   void AddLibraryItemToPlaylist(const QModelIndexList& indexes);
   void LibraryItemDoubleClicked(const QModelIndex& index);
@@ -147,8 +161,8 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void UpdateTrackPosition();
 
   void RadioDoubleClick(const QModelIndex& index);
-  void InsertRadioItem(RadioItem*);
-  void InsertRadioItems(const PlaylistItemList& items);
+  void InsertRadioItem(RadioItem*, bool clear_first);
+  void InsertRadioItems(const PlaylistItemList& items, bool clear_first);
   void ScrobblingEnabledChanged(bool value);
   void LastFMButtonVisibilityChanged(bool value);
   void Love();
@@ -182,18 +196,25 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   void EnsureEditTagDialogCreated();
   void OpenSettingsDialog();
   void OpenSettingsDialogAtPage(SettingsDialog::Page page);
+  void ShowSongInfoConfig();
+
+  void SaveGeometry();
+
+  void AddSongInfoGenerator(smart_playlists::GeneratorPtr gen);
+
+  void DeleteFinished(const SongList& songs_with_errors);
 
  private:
-  void SaveGeometry();
   void AddFilesToPlaylist(bool clear_first, const QList<QUrl>& urls);
   void AddLibraryItemToPlaylist(bool clear_first, const QModelIndexList& indexes);
   void AddLibrarySongsToPlaylist(bool clear_first, const SongList& songs);
+  void AddSmartPlaylistToPlaylist(bool clear_first, smart_playlists::GeneratorPtr gen);
   void AddDeviceSongsToPlaylist(bool clear_first, const SongList& songs);
   void AddUrls(bool play_now, const QList<QUrl>& urls);
+  void ConnectInfoView(SongInfoBase* view);
 
  private:
   Ui_MainWindow* ui_;
-  NetworkAccessManager* network_;
 
   SystemTrayIcon* tray_icon_;
   OSD* osd_;
@@ -205,13 +226,19 @@ class MainWindow : public QMainWindow, public PlatformInterface {
   RadioModel* radio_model_;
   PlaylistBackend* playlist_backend_;
   PlaylistManager* playlists_;
-  PlaylistParser* playlist_parser_;
   Player* player_;
   Library* library_;
   GlobalShortcuts* global_shortcuts_;
 
   DeviceManager* devices_;
 
+  LibraryViewContainer* library_view_;
+  FileView* file_view_;
+  RadioViewContainer* radio_view_;
+  DeviceView* device_view_;
+  SongInfoView* song_info_view_;
+  ArtistInfoView* artist_info_view_;
+
   boost::scoped_ptr<SettingsDialog> settings_dialog_;
   boost::scoped_ptr<AddStreamDialog> add_stream_dialog_;
   boost::scoped_ptr<AlbumCoverManager> cover_manager_;
@@ -248,6 +275,8 @@ class MainWindow : public QMainWindow, public PlatformInterface {
 
   bool was_maximized_;
   bool autoclear_playlist_;
+
+  BackgroundStreams* background_streams_;
 };
 
 #endif // MAINWINDOW_H
