@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,15 +16,18 @@
 */
 
 #include "config.h"
+#include "core/backgroundstreams.h"
 #include "iconloader.h"
 #include "mainwindow.h"
 #include "settingsdialog.h"
-#include "ui_settingsdialog.h"
 #include "engines/enginebase.h"
 #include "playlist/playlistview.h"
+#include "songinfo/songinfotextview.h"
 #include "widgets/osd.h"
 #include "widgets/osdpretty.h"
 
+#include "ui_settingsdialog.h"
+
 #ifdef ENABLE_WIIMOTEDEV
 #include "ui/wiimotedevshortcutsconfig.h"
 #include "ui_wiimotedevshortcutsconfig.h"
@@ -36,16 +40,56 @@
 
 #include <QColorDialog>
 #include <QDir>
+#include <QFontDialog>
 #include <QSettings>
 
 #include <QtDebug>
 
-SettingsDialog::SettingsDialog(QWidget* parent)
+void SettingsDialog::AddStream(const QString& name) {
+  QGroupBox* box = new QGroupBox(tr(name.toUtf8()));
+  QSlider* slider = new QSlider(Qt::Horizontal, box);
+  QCheckBox* check = new QCheckBox(box);
+  QHBoxLayout* layout = new QHBoxLayout(box);
+  layout->addWidget(slider);
+  layout->addWidget(check);
+
+  QVBoxLayout* streams_layout = qobject_cast<QVBoxLayout*>(
+        ui_->streams_page->layout());
+  streams_layout->insertWidget(streams_layout->count() - 1, box);
+
+  slider->setProperty("stream_name", name);
+  check->setProperty("stream_name", name);
+
+  connect(slider, SIGNAL(valueChanged(int)), SLOT(StreamVolumeChanged(int)));
+  connect(check, SIGNAL(toggled(bool)), SLOT(EnableStream(bool)));
+
+  slider->setValue(streams_->GetStreamVolume(name));
+  check->setCheckState(streams_->IsPlaying(name) ? Qt::Checked : Qt::Unchecked);
+}
+
+void SettingsDialog::EnableStream(bool enabled) {
+  const QString name = sender()->property("stream_name").toString();
+  streams_->EnableStream(name, enabled);
+}
+
+void SettingsDialog::StreamVolumeChanged(int value) {
+  const QString name = sender()->property("stream_name").toString();
+  streams_->SetStreamVolume(name, value);
+}
+
+void SettingsDialog::AddStreams() {
+  foreach (const QString& name, streams_->streams()) {
+    AddStream(name);
+  }
+}
+
+SettingsDialog::SettingsDialog(BackgroundStreams* streams, QWidget* parent)
   : QDialog(parent),
     gst_engine_(NULL),
     ui_(new Ui_SettingsDialog),
     loading_settings_(false),
-    pretty_popup_(new OSDPretty(OSDPretty::Mode_Draggable))
+    pretty_popup_(new OSDPretty(OSDPretty::Mode_Draggable)),
+    streams_(streams)
 {
   ui_->setupUi(this);
   pretty_popup_->SetMessage(tr("OSD Preview"), tr("Drag to reposition"),
@@ -53,9 +97,13 @@ SettingsDialog::SettingsDialog(QWidget* parent)
 
   // Icons
   ui_->list->item(Page_Playback)->setIcon(IconLoader::Load("media-playback-start"));
+  ui_->list->item(Page_SongInformation)->setIcon(IconLoader::Load("view-media-lyrics"));
   ui_->list->item(Page_GlobalShortcuts)->setIcon(IconLoader::Load("input-keyboard"));
   ui_->list->item(Page_Notifications)->setIcon(IconLoader::Load("help-hint"));
   ui_->list->item(Page_Library)->setIcon(IconLoader::Load("folder-sound"));
+  ui_->list->item(Page_BackgroundStreams)->setIcon(QIcon(":/icons/32x32/weather-showers-scattered.png"));
+
+  AddStreams();
 
 #ifdef ENABLE_WIIMOTEDEV
   // Wiimotedev page
@@ -115,6 +163,13 @@ SettingsDialog::SettingsDialog(QWidget* parent)
   qStableSort(names);
   ui_->language->addItems(names);
 
+  // Song info
+  QFile song_info_preview(":/lumberjacksong.txt");
+  song_info_preview.open(QIODevice::ReadOnly);
+  ui_->song_info_font_preview->setText(QString::fromUtf8(song_info_preview.readAll()));
+
+  connect(ui_->song_info_font_size, SIGNAL(valueChanged(double)), SLOT(SongInfoFontSizeChanged(double)));
+
   // Global shortcuts
 #ifdef Q_OS_MAC
   if (QSysInfo::MacintoshVersion != QSysInfo::MV_SNOWLEOPARD) {
@@ -153,7 +208,8 @@ SettingsDialog::SettingsDialog(QWidget* parent)
   ui_->list->setMinimumWidth(ui_->list->sizeHintForColumn(0));
 
 #ifdef Q_OS_DARWIN
-  ui_->list->item(Page_Behaviour)->setHidden(true);  // Hide "Behaviour" on mac.
+  ui_->b_show_tray_icon_->setEnabled(false);
+  ui_->startup_group_->setEnabled(false);
 #endif
 }
 
@@ -200,6 +256,7 @@ void SettingsDialog::accept() {
 
   s.beginGroup(MainWindow::kSettingsGroup);
   s.setValue("showtray", ui_->b_show_tray_icon_->isChecked());
+  s.setValue("keeprunning", ui_->b_keep_running_->isChecked());
   s.setValue("startupbehaviour", int(behaviour));
   s.endGroup();
 
@@ -228,9 +285,18 @@ void SettingsDialog::accept() {
   s.setValue("rgmode", ui_->replaygain_mode->currentIndex());
   s.setValue("rgpreamp", float(ui_->replaygain_preamp->value()) / 10 - 15);
   s.setValue("rgcompression", ui_->replaygain_compression->isChecked());
+  s.setValue("bufferduration", ui_->buffer_duration->value());
   s.endGroup();
 #endif
 
+  // Song info
+  s.beginGroup(SongInfoTextView::kSettingsGroup);
+  s.setValue("font_size", ui_->song_info_font_preview->font().pointSizeF());
+  s.endGroup();
+
+  ui_->lyric_settings->Save();
+
+  // Wii remotes
 #ifdef ENABLE_WIIMOTEDEV
   s.beginGroup(WiimotedevShortcuts::kActionsGroup);
   s.remove("");
@@ -274,6 +340,8 @@ void SettingsDialog::accept() {
   ui_->magnatune->Save();
   ui_->global_shortcuts->Save();
 
+  streams_->SaveStreams();
+
   QDialog::accept();
 }
 
@@ -300,6 +368,9 @@ void SettingsDialog::showEvent(QShowEvent*) {
   // Behaviour
   s.beginGroup(MainWindow::kSettingsGroup);
   ui_->b_show_tray_icon_->setChecked(s.value("showtray", true).toBool());
+  ui_->b_keep_running_->setChecked(s.value("keeprunning",
+      ui_->b_show_tray_icon_->isChecked()).toBool());
+
   MainWindow::StartupBehaviour behaviour = MainWindow::StartupBehaviour(
       s.value("startupbehaviour", MainWindow::Startup_Remember).toInt());
   switch (behaviour) {
@@ -317,6 +388,14 @@ void SettingsDialog::showEvent(QShowEvent*) {
     ui_->language->setCurrentIndex(ui_->language->findText(name));
   s.endGroup();
 
+  // Song Info
+  s.beginGroup(SongInfoTextView::kSettingsGroup);
+  ui_->song_info_font_size->setValue(
+      s.value("font_size", SongInfoTextView::kDefaultFontSize).toReal());
+  s.endGroup();
+
+  ui_->lyric_settings->Load();
+
   // Last.fm
   ui_->lastfm->Load();
 
@@ -353,6 +432,7 @@ void SettingsDialog::showEvent(QShowEvent*) {
   ui_->replaygain_mode->setCurrentIndex(s.value("rgmode", 0).toInt());
   ui_->replaygain_preamp->setValue(s.value("rgpreamp", 0.0).toDouble() * 10 + 150);
   ui_->replaygain_compression->setChecked(s.value("rgcompression", true).toBool());
+  ui_->buffer_duration->setValue(s.value("bufferduration", 1000).toInt());
   s.endGroup();
 #endif
 
@@ -501,3 +581,14 @@ void SettingsDialog::OpenAtPage(Page page) {
   ui_->list->setCurrentRow(page);
   show();
 }
+
+void SettingsDialog::SetSongInfoView(SongInfoView* view) {
+  ui_->lyric_settings->SetSongInfoView(view);
+}
+
+void SettingsDialog::SongInfoFontSizeChanged(qreal value) {
+  QFont font;
+  font.setPointSizeF(value);
+
+  ui_->song_info_font_preview->setFont(font);
+}
