@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -41,6 +42,8 @@ class AutoExpandingTreeView : public QTreeView {
 
  private slots:
   void ItemExpanded(const QModelIndex& index);
+  void ItemClicked(const QModelIndex& index);
+  void ItemDoubleClicked(const QModelIndex& index);
 
  private:
   bool RecursivelyExpand(const QModelIndex& index, int* count);
@@ -48,6 +51,8 @@ class AutoExpandingTreeView : public QTreeView {
  private:
   bool auto_open_;
   bool expand_on_reset_;
+
+  bool ignore_next_click_;
 };
 
 #endif // AUTOEXPANDINGTREEVIEW_H
