@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,7 +24,6 @@
 #include <QWidget>
 
 class AlbumCoverLoader;
-class NetworkAccessManager;
 
 class QActionGroup;
 class QMenu;
@@ -52,7 +52,6 @@ public:
     LargeSongDetails = 1,
   };
 
-  void set_network(NetworkAccessManager* network) { network_ = network; }
   void set_ideal_height(int height);
 
   bool show_above_status_bar() const;
@@ -66,6 +65,7 @@ public slots:
   void NowPlaying(const Song& metadata);
   void Stopped();
   void AllHail(bool hypnotoad);
+  void EnableKittens(bool aww);
 
 protected:
   void paintEvent(QPaintEvent* e);
@@ -88,12 +88,12 @@ private:
   void CreateModeAction(Mode mode, const QString& text, QActionGroup* group,
                         QSignalMapper* mapper);
   void UpdateDetailsText();
-  void UpdateHeight();
+  void UpdateHeight(AlbumCoverLoader* loader);
   void DrawContents(QPainter* p);
 
 private:
   BackgroundThread<AlbumCoverLoader>* cover_loader_;
-  NetworkAccessManager* network_;
+  BackgroundThread<AlbumCoverLoader>* kitten_loader_;
 
   Mode mode_;
 
@@ -119,6 +119,8 @@ private:
 
   static const char* kHypnotoadPath;
   QMovie* hypnotoad_;
+
+  bool aww_;
 };
 
 #endif // NOWPLAYINGWIDGET_H
