@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,7 +27,6 @@
 #include "core/backgroundthread.h"
 #include "core/song.h"
 
-class NetworkAccessManager;
 class OrgFreedesktopNotificationsInterface;
 class OSDPretty;
 class SystemTrayIcon;
@@ -45,7 +45,7 @@ class OSD : public QObject {
   Q_OBJECT
 
  public:
-  OSD(SystemTrayIcon* tray_icon, NetworkAccessManager* network, QObject* parent = 0);
+  OSD(SystemTrayIcon* tray_icon, QObject* parent = 0);
   ~OSD();
 
   static const char* kSettingsGroup;
@@ -103,6 +103,7 @@ class OSD : public QObject {
  private slots:
   void CallFinished(QDBusPendingCallWatcher* watcher);
   void CoverLoaderInitialised();
+  void CoverArtPathReady(const Song& song, const QString& image_path);
   void AlbumArtLoaded(quint64 id, const QImage& image);
   void AlbumArtLoaded(const WaitingForAlbumArt info, const QImage& image);
 
@@ -118,7 +119,6 @@ class OSD : public QObject {
 
   OSDPretty* pretty_popup_;
 
-  NetworkAccessManager* network_;
   BackgroundThread<AlbumCoverLoader>* cover_loader_;
   QMap<quint64, WaitingForAlbumArt> waiting_for_album_art_;
 
