@@ -1,4 +1,5 @@
 /* This file is part of Clementine.
+   Copyright 2010, David Sansome <me@davidsansome.com>
 
    Clementine is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -94,6 +95,18 @@ void StretchHeaderView::UpdateWidths(const QList<int>& sections) {
 }
 
 void StretchHeaderView::HideSection(int logical) {
+  // Would this hide the last section?
+  bool all_hidden = true;
+  for (int i=0 ; i<count() ; ++i) {
+    if (i != logical && !isSectionHidden(i) && sectionSize(i) > 0) {
+      all_hidden = false;
+      break;
+    }
+  }
+  if (all_hidden) {
+    return;
+  }
+
   if (!stretch_enabled_) {
     hideSection(logical);
     return;
